import { JSX } from "./jsx.js";
import { bgColor, buildRend, buildSeance, datasAvailable, icon, openPage, shadowRoots, timeFmt } from "./webapp.js";
import { now } from "./webapp.js";
import { styleLinkTag } from "./commons.js";
export class Calendar extends HTMLElement {
    constructor() {
        super(...arguments);
        this.oneColDays = null;
    }
    async connectedCallback() {
        var _a;
        if (this.shadowRoot)
            return;
        const sr = this.attachShadow({ mode: "open" });
        shadowRoots.add(sr);
        sr.appendChild(await styleLinkTag("calendar.css"));
        this.weeks = JSX.createElement("div", null);
        sr.append(this.weeks);
        this.icalElt = JSX.createElement("div", { id: "icalArea" },
            JSX.createElement("a", { id: "icalLink", href: "ical/session.ics", target: "_blank", title: "Agenda au format iCalendar (.ics)", download: "session.ics" },
                "Agenda au format ",
                JSX.createElement("i", null, "iCalendar (.ics)")),
            JSX.createElement("button", { id: "icalCopy", onclick: this.onIcalCopy, title: "Copier l'url de l'agenda au format iCalendar (.ics)" }, "Copier l'url"));
        const timedDatas = await datasAvailable;
        const showDays = [];
        for (const d of timedDatas.data) {
            if (d.T === "sea") {
                const start = new Date(d.start);
                if (start.getTime() > 0) {
                    showDays[start.getDay()] = true;
                    const end = new Date(d.end);
                    if (!(start < end)) {
                        end.setTime(start.getTime());
                        end.setHours(end.getHours() + 1);
                    }
                    this.getWeekFor(start).addSea(d, start, end);
                }
            }
            else if (d.T === "seq") {
                const seq = d;
                let start = new Date(seq.startOrig);
                if (start.getTime() > 0) {
                    showDays[start.getDay()] = true;
                    let end = new Date(seq.endOrig);
                    if (!(start < end))
                        end = start;
                    showDays[getEndDay(end)] = true;
                    let w = this.getWeekFor(start);
                    do {
                        if (w.end >= end) {
                            w.addSeq(d, start, end);
                            break;
                        }
                        w.addSeq(d, start, w.end);
                        start = w.end;
                        w = this.getWeekFor(start);
                    } while (true);
                }
            }
            else if ((_a = d.rend) === null || _a === void 0 ? void 0 : _a.limit) {
                const r = d.rend;
                const dt = new Date(r.limit);
                if (dt.getTime() > 0) {
                    showDays[dt.getDay()] = true;
                    this.getWeekFor(dt).addRend(r, dt);
                }
            }
        }
        let showCols = showDays;
        if (firstDayOfWeek !== 0) {
            showCols = [];
            const diff = 7 - firstDayOfWeek;
            for (let i = 0; i < diff; i++)
                showCols[i] = showDays[i + firstDayOfWeek];
            for (let i = diff; i < 7; i++)
                showCols[i] = showDays[i - diff];
        }
        let countCols;
        if (firstDayOfWeek === 1) {
            countCols = 0;
            const firstCol = showCols.indexOf(true);
            if (firstCol < 5) {
                for (let i = 0; i < 5; i++)
                    showCols[i] = true;
                countCols = 5;
            }
            if (showCols[5] || showCols[6]) {
                showCols[5] = showCols[6] = true;
                countCols += 2;
            }
        }
        else {
            const firstCol = showCols.indexOf(true);
            const lastCol = showCols.lastIndexOf(true);
            countCols = 1;
            for (let i = firstCol; i < lastCol; i++) {
                countCols++;
                showCols[i] = true;
            }
        }
        this.showCols = showCols;
        let prev = null;
        let w = this.weeks.firstElementChild;
        let prevMonth = 99;
        let prevYear = 9999;
        while (w) {
            if (w instanceof Week) {
                if (prev && prev.weekOfYear !== w.weekOfYear - 1 && prev.weekOfYear !== w.weekOfYear) {
                    w.parentNode.insertBefore(JSX.createElement("div", { class: "rowSep" }), w);
                }
                prev = w;
                if (w.start.getMonth() !== prevMonth || prevYear != w.start.getFullYear()) {
                    const monthElt = JSX.createElement("div", { class: "monthLabel" }, prevYear != w.start.getFullYear() ? monthYearFmt.format(w.start) : monthFmt.format(w.start));
                    const firstMonthDay = new Date(w.start);
                    firstMonthDay.setDate(1);
                    monthElt.date = firstMonthDay.getTime();
                    w.insertAdjacentElement("beforebegin", monthElt);
                    prevMonth = w.start.getMonth();
                    prevYear = w.start.getFullYear();
                }
            }
            w = w.nextElementSibling;
        }
        this.setAttribute("count-cols", countCols.toString());
        const sizeLstn = matchMedia(`(max-width: ${countCols * 7.5}em)`);
        if (sizeLstn.matches)
            this.buildOneCol();
        else
            this.buildWeeks();
        this.setToday(true);
        sizeLstn.onchange = (ev) => {
            if (ev.matches)
                this.buildOneCol();
            else
                this.buildWeeks();
            this.setToday(false);
        };
    }
    refresh() {
        this.setToday(false, true);
    }
    setToday(scrollTo, refresh) {
        var _a;
        const today = new Date(now());
        const tsNow = today.getTime();
        today.setHours(0, 0, 0, 0);
        const tsDay = today.getTime();
        setTimeout(this.setToday.bind(this, false, true), tsDay + 86401000 - tsNow);
        (_a = this.todayMark) === null || _a === void 0 ? void 0 : _a.remove();
        let nodeToScroll;
        if (!this.weeks.hidden) {
            let w = this.weeks.firstElementChild;
            while (w) {
                if (w instanceof Week) {
                    if (w.setToday(tsDay, refresh) === 0)
                        nodeToScroll = w;
                }
                if (!nodeToScroll && w.date > tsDay) {
                    this.todayMark = nodeToScroll = JSX.createElement("div", { class: "day today" },
                        "Aujourd'hui, ",
                        firstFmt.format(new Date(tsDay)));
                    w.insertAdjacentElement("beforebegin", nodeToScroll);
                }
                w = w.nextElementSibling;
            }
        }
        else if (this.oneColDays) {
            let d = this.oneColDays.firstElementChild;
            while (d) {
                const t = d.date;
                if (t === tsDay) {
                    d.classList.toggle("today", true);
                    nodeToScroll = d;
                }
                else {
                    if (!nodeToScroll && t > tsDay) {
                        this.todayMark = nodeToScroll = JSX.createElement("div", { class: "day today" },
                            "Aujourd'hui, ",
                            firstFmt.format(new Date(tsDay)));
                        d.insertAdjacentElement("beforebegin", nodeToScroll);
                    }
                    if (refresh)
                        d.classList.toggle("today", false);
                }
                d = d.nextElementSibling;
            }
        }
        if (scrollTo && nodeToScroll)
            nodeToScroll.scrollIntoView({ behavior: "smooth", block: "center" });
    }
    buildOneCol() {
        this.weeks.hidden = true;
        if (!this.oneColDays)
            this.oneColDays = this.shadowRoot.appendChild(JSX.createElement("div", { id: "oneCol" }));
        else
            this.oneColDays.textContent = null;
        for (let w = this.weeks.firstElementChild; w; w = w.nextElementSibling)
            if (w instanceof Week)
                w.buildAsCol(this.oneColDays);
        this.oneColDays.hidden = false;
        this.oneColDays.insertAdjacentElement("afterend", this.icalElt);
    }
    buildWeeks() {
        if (this.oneColDays)
            this.oneColDays.hidden = true;
        this.weeks.hidden = false;
        for (let w = this.weeks.firstElementChild; w; w = w.nextElementSibling)
            if (w instanceof Week)
                w.buildAsRow(this.showCols);
        this.weeks.insertAdjacentElement("beforebegin", this.icalElt);
    }
    getWeekFor(dt) {
        let w = this.weeks.firstElementChild;
        while (w) {
            if (w instanceof Week && w.end > dt) {
                if (w.start <= dt)
                    return w;
                return this.weeks.insertBefore(new Week().init(dt), w);
            }
            w = w.nextElementSibling;
        }
        return this.weeks.appendChild(new Week().init(dt));
    }
    onIcalCopy() {
        navigator.clipboard.writeText(new URL("ical/session.ics", document.baseURI).href).then(() => {
            const done = JSX.createElement("span", { id: "icalCopyDone" }, "L'url a été copiée dans le presse-papier.");
            this.insertAdjacentElement("afterend", done);
            setTimeout(() => { done.remove(); }, 3000);
        });
    }
}
customElements.define('p-cal', Calendar);
class Week extends HTMLElement {
    constructor() {
        super(...arguments);
        this.seqs = null;
        this.days = [];
    }
    init(dtIn) {
        this.start = new Date(dtIn);
        this.start.setDate(Math.max(1, dtIn.getDate() - dayOfWeek(dtIn.getDay())));
        this.start.setHours(0);
        this.start.setMinutes(0);
        this.date = this.start.getTime();
        this.weekStart = this.start;
        if (this.start.getDate() === 1) {
            const delta = dayOfWeek(this.start.getDay());
            if (delta > 0) {
                this.weekStart = new Date(this.start);
                this.weekStart.setDate(this.weekStart.getDate() - delta);
            }
        }
        this.weekOfYear = weekOfYear(this.weekStart);
        this.end = new Date(this.weekStart);
        this.end.setDate(this.weekStart.getDate() + 7);
        if (this.start.getDate() > this.end.getDate())
            this.end.setDate(1);
        return this;
    }
    setToday(today, refresh) {
        if (this.start.getTime() <= today && this.end.getTime() > today) {
            this.classList.toggle('currWeek', true);
            if (this.front) {
                let d = this.front.firstElementChild;
                while (d) {
                    if (d.date === today) {
                        d.classList.toggle('today', true);
                    }
                    else if (refresh) {
                        d.classList.toggle('today', false);
                    }
                    d = d.nextElementSibling;
                }
            }
            return 0;
        }
        else if (refresh && this.classList.contains('currWeek')) {
            this.classList.toggle('currWeek', false);
            if (this.front) {
                let d = this.front.firstElementChild;
                while (d) {
                    d.classList.toggle('today', false);
                    d = d.nextElementSibling;
                }
            }
        }
        return today < this.weekStart.getTime() ? -1 : 1;
    }
    buildAsRow(showCols) {
        this.textContent = null;
        this.appendChild(JSX.createElement("div", { class: "numWeek", title: "Numéro annuel de la semaine" }, this.weekOfYear));
        const back = this.appendChild(JSX.createElement("div", { class: "back" }));
        const front = this.front = this.appendChild(JSX.createElement("div", { class: "front" }));
        if (this.seqs)
            for (const seq of this.seqs)
                front.appendChild(seq);
        const dt = new Date(this.weekStart);
        const mapDaysColNumber = [];
        let colNumber = 1;
        const firstSeq = front.firstElementChild;
        for (let i = 0; i < 7; i++) {
            mapDaysColNumber[i] = colNumber;
            if (i > 0)
                dt.setDate(dt.getDate() + 1);
            if (showCols[i]) {
                back.appendChild(JSX.createElement("div", { class: "day" }));
                if (dt >= this.start && dt < this.end) {
                    const d = front.insertBefore(JSX.createElement("div", { class: "day" }, colNumber === 1 || dt.getDate() === 1 ? firstFmt.format(dt) : nextFmt.format(dt)), firstSeq);
                    d.date = dt.getTime();
                    d.style.gridColumn = colNumber.toString();
                    const dayCtn = front.appendChild(this.days[i] || JSX.createElement("div", { class: "dayCtn" }));
                    dayCtn.style.gridColumn = colNumber.toString();
                }
                colNumber++;
            }
        }
        for (let s = firstSeq; s; s = s.nextElementSibling) {
            if (!('from' in s))
                break;
            s.style.gridColumnStart = mapDaysColNumber[dayOfWeek(s.from.getDay())].toString();
            s.style.gridColumnEnd = (mapDaysColNumber[dayOfWeek(getEndDay(s.to))] + 1).toString();
        }
        const tplRows = `repeat(${colNumber - 1}, 1fr)`;
        back.style.gridTemplateColumns = tplRows;
        front.style.gridTemplateColumns = tplRows;
    }
    buildAsCol(col) {
        for (let i = 0; i < this.days.length; i++) {
            const day = this.days[i];
            if (day) {
                const dt = new Date(this.start);
                if (i > 0)
                    dt.setDate(dt.getDate() + i);
                const dayHead = JSX.createElement("div", { class: "day" }, firstFmt.format(dt));
                dayHead.date = dt.getTime();
                col.append(dayHead, day);
            }
        }
    }
    addSeq(d, from, to) {
        const seq = JSX.createElement("div", { class: "evt seq" });
        seq.innerHTML = d.ti;
        seq.from = from;
        seq.to = to;
        seq.style.backgroundImage = `url(${icon(d)})`;
        seq.style.backgroundColor = bgColor(d);
        seq.addEventListener("click", () => { openPage(d.url); });
        if (!this.seqs)
            this.seqs = [seq];
        else
            this.seqs.push(seq);
    }
    addSea(d, start, end) {
        this.addEvtInDay(start, buildSeance(d, start, end, timeFmt));
    }
    addRend(d, dt) {
        this.addEvtInDay(dt, buildRend(d, dt, timeFmt));
    }
    addEvtInDay(start, view) {
        const day = dayOfWeek(start.getDay());
        const ctn = this.days[day] || (this.days[day] = JSX.createElement("div", { class: "dayCtn" }));
        let before = null;
        for (let other = ctn.firstElementChild; other; other = other.nextElementSibling) {
            if (other.start >= start) {
                before = other;
                break;
            }
        }
        view.start = start;
        ctn.insertBefore(view, before);
        return view;
    }
}
customElements.define('p-week', Week);
export const firstFmt = Intl.DateTimeFormat(undefined, { month: "short", day: 'numeric', weekday: 'short' });
export const nextFmt = Intl.DateTimeFormat(undefined, { day: 'numeric', weekday: 'short' });
export const monthFmt = Intl.DateTimeFormat(undefined, { month: "long" });
export const monthYearFmt = Intl.DateTimeFormat(undefined, { month: "long", year: 'numeric' });
function getEndDay(dt) {
    return dt.getHours() === 0 && dt.getMinutes() === 0 ? (6 + dt.getDay()) % 7 : dt.getDay();
}
let firstDayOfWeek = 1;
function dayOfWeek(day) {
    return day < firstDayOfWeek ? 7 + day - firstDayOfWeek : day - firstDayOfWeek;
}
function isoDayOfWeek(day) {
    return day === 0 ? 6 : day - 1;
}
function weekOfYear(d) {
    const firstDay = new Date(d.getFullYear(), 0, 1);
    const firstDayOfWeek = isoDayOfWeek(firstDay.getDay());
    const distInMs = d.getTime() - firstDay.getTime() - (d.getTimezoneOffset() - firstDay.getTimezoneOffset()) * 60000;
    const distInDay = Math.floor(distInMs / 86400000);
    let numWeek = Math.floor((distInDay + firstDayOfWeek) / 7);
    if (firstDayOfWeek < 4) {
        numWeek++;
        if (numWeek > 52) {
            const nextYear = new Date(d.getFullYear() + 1, 0, 1);
            numWeek = isoDayOfWeek(nextYear.getDay()) < 4 ? 1 : 53;
        }
    }
    return numWeek;
}
