import { JSX, ShadowJsx } from "./jsx.js";
import { DIALOG } from "./dialog.js";
window.popstateFromUs = false;
export function makeButton(elt, title) {
    elt.onclick = elt.onClick;
    elt.addEventListener("keypress", function (ev) {
        if (ev.key === " " || ev.key === "Enter")
            ev.target.click();
    });
    Promise.resolve().then(() => {
        elt.setAttribute("role", "button");
        elt.tabIndex = 0;
        if (title)
            elt.title = title;
    });
}
export class RendButton extends HTMLElement {
    constructor() {
        super();
        makeButton(this);
    }
    async onClick(ev) {
        const rend = new (await import("./rend.js")).RendArea();
        rend.setAttribute("href", this.getAttribute("href"));
        const popup = document.body.appendChild(document.createElement("dialog"));
        popup.appendChild(rend);
        let lastPointerDown;
        popup.addEventListener("pointerdown", function (ev) {
            lastPointerDown = ev.target;
        });
        popup.addEventListener("click", function (ev) {
            if (ev.target instanceof HTMLDialogElement && ev.target === lastPointerDown)
                popup.firstElementChild.tryClose();
        });
        popup.showModal();
    }
}
customElements.define('p-rendbutton', RendButton);
export class CompactToggle extends HTMLElement {
    constructor() {
        super();
        makeButton(this);
    }
    get enabled() {
        return this.hasAttribute("enabled");
    }
    set enabled(enabled) {
        if (enabled)
            this.setAttribute("enabled", "");
        else
            this.removeAttribute("enabled");
        this.setNextsHidden(enabled);
        this.parentElement.classList.toggle("masked", enabled);
        const root = this.closest(".page").querySelector(".content");
        if (enabled)
            this.compactNode(root);
        else
            this.uncompactNode(root);
    }
    compactNode(root) {
        const isSeance = root.classList.contains("seance");
        let firstCt;
        let hasBody = false;
        const tw = document.createTreeWalker(root, NodeFilter.SHOW_ELEMENT, {
            acceptNode: function (n) {
                if (n.classList.contains("bk")) {
                    if (!firstCt)
                        firstCt = n;
                    hasBody = true;
                    n.hidden = true;
                    return NodeFilter.FILTER_REJECT;
                }
                if (n.classList.contains("sub") || n.classList.contains("subPg")) {
                    if (!firstCt)
                        firstCt = n;
                    if (isSeance) {
                        hasBody = true;
                        n.hidden = true;
                    }
                    return NodeFilter.FILTER_ACCEPT;
                }
                if (n.classList.contains("abstract") || n.classList.contains("details"))
                    return NodeFilter.FILTER_SKIP;
                return NodeFilter.FILTER_REJECT;
            }
        });
        if (tw.firstChild()) {
            do {
                this.compactNode(tw.currentNode);
            } while (tw.nextSibling());
        }
        if (hasBody) {
            firstCt.insertAdjacentElement("beforebegin", JSX.createElement("button", { class: "showBody closed", title: "Afficher et masquer les détails...", onclick: showBtnClick }));
        }
    }
    uncompactNode(root) {
        const isSeance = root.classList.contains("seance");
        const rem = [];
        const tw = document.createTreeWalker(root, NodeFilter.SHOW_ELEMENT, {
            acceptNode: function (n) {
                if (n.classList.contains("bk")) {
                    n.hidden = false;
                    return NodeFilter.FILTER_REJECT;
                }
                if (n.classList.contains("showBody")) {
                    rem.push(n);
                    return NodeFilter.FILTER_REJECT;
                }
                if (n.classList.contains("sub") || n.classList.contains("subPg")) {
                    if (isSeance)
                        n.hidden = false;
                    return NodeFilter.FILTER_ACCEPT;
                }
                if (n.classList.contains("abstract") || n.classList.contains("details"))
                    return NodeFilter.FILTER_SKIP;
                return NodeFilter.FILTER_REJECT;
            }
        });
        while (tw.nextNode())
            this.uncompactNode(tw.currentNode);
        rem.forEach((n) => n.remove());
    }
    setNextsHidden(hidden) {
        for (let n = this.nextElementSibling; n; n = n.nextElementSibling)
            n.hidden = hidden;
    }
    connectedCallback() {
        var _a;
        const outline = this.parentElement;
        const lnk = (_a = this.findLastLnk()) === null || _a === void 0 ? void 0 : _a.getAttribute("href");
        if (!lnk) {
            outline.remove();
            return;
        }
        if (lnk[0] == '#') {
            const target = document.getElementById(lnk.substring(1));
            if (target)
                requestAnimationFrame(function () {
                    if (target.offsetParent && target.offsetTop < target.offsetParent.clientHeight) {
                        outline.remove();
                    }
                });
        }
        const label = this.appendChild(document.createElement("span"));
        label.className = "toggleLabel";
        label.append("Condensé");
        const toggle = this.appendChild(document.createElement("span"));
        toggle.className = "toggleFrame";
        const grip = toggle.appendChild(document.createElement("span"));
        grip.className = "toggleGrip";
    }
    findLastLnk() {
        let elt = this.parentElement.lastElementChild;
        while (elt && elt.localName !== "a")
            elt = elt.lastElementChild;
        return elt;
    }
    onClick() {
        this.enabled = !this.enabled;
    }
}
function showBtnClick(ev) {
    const showBody = this.classList.contains("closed");
    this.classList.toggle("closed", !showBody);
    this.classList.toggle("opened", showBody);
    const root = this.closest(".sub, .subPg, .content");
    const isSeance = root.classList.contains("seance");
    document.createTreeWalker(root, NodeFilter.SHOW_ELEMENT, {
        acceptNode: function (n) {
            if (n.classList.contains("bk") || (isSeance && (n.classList.contains("sub") || n.classList.contains("subPg")))) {
                n.hidden = !showBody;
            }
            else if (n.classList.contains("abstract") || n.classList.contains("details")) {
                return NodeFilter.FILTER_SKIP;
            }
            return NodeFilter.FILTER_REJECT;
        }
    }).nextNode();
}
customElements.define('p-compact-toggle', CompactToggle);
export class InfoButton extends HTMLElement {
    constructor() {
        super();
        makeButton(this);
    }
    async onClick(ev) {
        ev.preventDefault();
        ev.stopImmediatePropagation();
        const ct = JSX.createElement("div", null,
            JSX.createElement(ShadowJsx, null,
                await styleLinkTag("info.css"),
                JSX.createElement("button", { id: "close", onclick: function () { DIALOG.findDialog(this).close(); } }),
                this.firstElementChild.content.cloneNode(true)));
        this.classList.add("on");
        DIALOG.showPopup(ct, this).addEventListener("close", () => { this.classList.remove("on"); });
    }
}
customElements.define('p-info', InfoButton);
export const onHideBars = new Set();
export class HideBarsButton extends HTMLElement {
    constructor() {
        super();
        makeButton(this, "Masquer les entêtes");
    }
    connectedCallback() {
        this.bar = this.parentElement;
        onHideBars.add(this);
    }
    disconnectedCallback() {
        onHideBars.delete(this);
    }
    onClick() {
        for (const lstn of onHideBars)
            lstn.onHideBars(true);
        document.body.appendChild(new RestoreBarsButton());
    }
    onHideBars(hide) {
        this.bar.hidden = hide;
    }
}
customElements.define('p-hidebars', HideBarsButton);
export class RestoreBarsButton extends HTMLElement {
    constructor() {
        super();
        makeButton(this, "Réafficher les entêtes");
    }
    onClick() {
        this.remove();
        for (const lstn of onHideBars)
            lstn.onHideBars(false);
    }
}
customElements.define('p-restorebars', RestoreBarsButton);
export async function styleLinkTag(cssFile) {
    const url = new URL(`${window.skinRoot}${cssFile}`, document.baseURI);
    let promise = styles.get(url.href);
    if (!promise) {
        const preload = JSX.createElement("link", { rel: "preload", as: "style", href: url.href });
        promise = new Promise((resolver) => {
            preload.onload = resolver;
            preload.onerror = resolver;
            document.head.appendChild(preload);
        });
        styles.set(url.href, promise);
    }
    await promise;
    return JSX.createElement("link", { rel: "stylesheet", href: url.href });
}
const styles = new Map();
