import { JSX } from "./jsx.js";
import { buildAct, buildRend, buildSeance, datasAvailable, dateTimeFmt, isDataAct, isDataSea, isDataTimedPeriod, now, parentOf, timeFmt, tsToLocalDt } from "./webapp.js";
import { styleLinkTag } from "./commons.js";
export class Desk extends HTMLElement {
    async connectedCallback() {
        const sr = this.attachShadow({ mode: "open" });
        sr.append(await styleLinkTag("desk.css"));
        this.todayRoot = JSX.createElement("div", { class: "list" });
        this.tomorrowRoot = JSX.createElement("div", { class: "list" });
        this.meetingsRoot = JSX.createElement("div", { class: "list" });
        this.rendsRoot = JSX.createElement("div", { class: "list" });
        sr.append(JSX.createElement("article", { class: "widget" },
            JSX.createElement("h2", null, "Aujourd'hui"),
            this.todayRoot), JSX.createElement("article", { class: "widget" },
            JSX.createElement("h2", null, "Demain"),
            this.tomorrowRoot), JSX.createElement("article", { class: "widget" },
            JSX.createElement("h2", null, "Prochains rendez-vous"),
            this.meetingsRoot), JSX.createElement("article", { class: "widget" },
            JSX.createElement("h2", null, "Prochains rendus"),
            this.rendsRoot));
        this.refresh();
    }
    async refresh() {
        var _a;
        if (!this.todayRoot)
            return;
        const timedDatas = await datasAvailable;
        const maxMeetings = 5;
        const maxRends = 5;
        const nextMeetings = [];
        const nextRends = [];
        const todayEvts = [];
        const tomorrowEvts = [];
        let todayMoreEvts;
        let tomorrowMoreEvts;
        const nowTs = now();
        const nowDt = tsToLocalDt(nowTs);
        const today = nowDt.substring(0, 10);
        const todayMin = today + "T00:00";
        const tom = new Date(nowTs);
        tom.setDate(tom.getDate() + 1);
        const tomorrow = tsToLocalDt(tom.getTime()).substring(0, 10);
        const tomorrowMin = tomorrow + "T00:00";
        function isInPeriod(d) {
            const endMin = d.end.substring(0, 16);
            if (endMin > tomorrowMin) {
                const start = d.start.substring(0, 10);
                if (start <= today)
                    return 3;
                if (start == tomorrow)
                    return 2;
            }
            else if (endMin > todayMin) {
                if (d.start.substring(0, 10) <= today)
                    return 1;
            }
            return 0;
        }
        let hasRend = false;
        for (const d of timedDatas.data) {
            if (d.T === "sea") {
                const start = d.start;
                if (start) {
                    addEvent(d, nextMeetings, (d) => d.start, nowDt, maxMeetings);
                    if (start.startsWith(today))
                        todayEvts.push(d);
                    else if (start.startsWith(tomorrow))
                        tomorrowEvts.push(d);
                }
            }
            else if (isDataAct(d) && !isDataSea(parentOf(d))) {
                const limit = (_a = d.rend) === null || _a === void 0 ? void 0 : _a.limit;
                if (limit) {
                    hasRend = true;
                    addEvent(d, nextRends, (d) => { var _a; return (_a = d.rend) === null || _a === void 0 ? void 0 : _a.limit; }, nowDt, maxRends);
                }
                let added = 0;
                if (limit === null || limit === void 0 ? void 0 : limit.startsWith(today)) {
                    todayEvts.push(d);
                    added = 1;
                }
                else if (limit === null || limit === void 0 ? void 0 : limit.startsWith(tomorrow)) {
                    tomorrowEvts.push(d);
                    added = 2;
                }
                if (!limit || limit > tomorrow) {
                    let p = parentOf(d);
                    while (p && !isDataTimedPeriod(p))
                        p = parentOf(p);
                    if (p && isDataTimedPeriod(p)) {
                        const inPeriod = isInPeriod(p);
                        if (added !== 1 && (inPeriod & 1) === 1) {
                            if (!todayMoreEvts)
                                todayMoreEvts = [d];
                            else
                                todayMoreEvts.push(d);
                        }
                        if (added !== 2 && (inPeriod & 2) === 2) {
                            if (!tomorrowMoreEvts)
                                tomorrowMoreEvts = [d];
                            else
                                tomorrowMoreEvts.push(d);
                        }
                    }
                }
            }
        }
        fillEvtsAtDate(todayEvts, todayMoreEvts, this.todayRoot, today);
        fillEvtsAtDate(tomorrowEvts, tomorrowMoreEvts, this.tomorrowRoot, tomorrow);
        if (nextMeetings.length === 0) {
            this.meetingsRoot.textContent = "Aucun rendez-vous planifié";
        }
        else {
            this.meetingsRoot.textContent = null;
            for (const d of nextMeetings) {
                this.meetingsRoot.appendChild(buildSeance(d, new Date(d.start), new Date(d.end), dateTimeFmt));
            }
        }
        this.rendsRoot.parentElement.hidden = !hasRend;
        if (hasRend) {
            if (nextRends.length === 0) {
                this.rendsRoot.textContent = "Aucun rendu à venir";
            }
            else {
                this.rendsRoot.textContent = null;
                for (const d of nextRends) {
                    const r = d.rend;
                    this.rendsRoot.appendChild(buildRend(r, new Date(r.limit), dateTimeFmt));
                }
            }
        }
    }
}
customElements.define('p-desk', Desk);
function addEvent(d, list, getDt, after, maxInList) {
    const dt = getDt(d);
    if (!dt || dt < after)
        return;
    const idx = list.findIndex((item) => getDt(item) > dt);
    if (idx < 0) {
        if (list.length < maxInList)
            list.push(d);
    }
    else {
        list.splice(idx, 0, d);
        if (list.length > maxInList)
            list.length = maxInList;
    }
}
function fillEvtsAtDate(list, list2, root, day) {
    if (list.length === 0 && !list2) {
        root.textContent = "Rien de prévu";
    }
    else {
        root.textContent = null;
        function draw(d) {
            var _a;
            if (isDataAct(d)) {
                const r = d.rend;
                if (r)
                    root.appendChild(buildRend(r, r.limit ? new Date(r.limit) : null, ((_a = r.limit) === null || _a === void 0 ? void 0 : _a.startsWith(day)) ? timeFmt : dateTimeFmt));
                else
                    root.appendChild(buildAct(d));
            }
            else {
                root.appendChild(buildSeance(d, new Date(d.start), new Date(d.end), timeFmt));
            }
        }
        list.forEach(draw);
        if (list2) {
            const max = 12 - list.length;
            if (list2.length > max)
                list2.length = Math.max(0, max);
            list2.forEach(draw);
        }
    }
}
