import { JSX } from "./jsx.js";
import { styleLinkTag } from "./commons.js";
export var DIALOG;
(function (DIALOG) {
    async function confirm(msg, okLabel = "Ok", cancelLabel = "Annuler") {
        let resolve;
        const promise = new Promise((r) => resolve = r);
        const dialog = document.body.appendChild(JSX.createElement("dialog", { class: "confirmDialog" }));
        const root = dialog.appendChild(JSX.createElement("div", { class: "confirm" }));
        const sr = root.attachShadow({ mode: "open" });
        sr.append(await styleLinkTag("dialog.css"), JSX.createElement("section", { id: "msg" }, msg), JSX.createElement("section", { id: "bottomBar" },
            JSX.createElement("button", { type: "submit", id: "ok" }, okLabel),
            JSX.createElement("button", { id: "cancel" }, cancelLabel)));
        sr.addEventListener("click", function (ev) {
            dialog.close(ev.target.id);
        });
        showDialogModal(dialog);
        dialog.addEventListener("close", function () { resolve(this.returnValue === "ok"); });
        return promise;
    }
    DIALOG.confirm = confirm;
    async function showNotif(msg) {
        const dialog = (stackModal[stackModal.length - 1] || document.body).appendChild(JSX.createElement("dialog", { class: "notifDialog" }));
        const root = dialog.appendChild(JSX.createElement("div", { class: "notif" }));
        const sr = root.attachShadow({ mode: "open" });
        sr.append(await styleLinkTag("dialog.css"), JSX.createElement("section", { id: "msg" }, msg));
        dialog.show();
        setTimeout(() => {
            dialog.close();
            dialog.remove();
        }, 5000);
    }
    DIALOG.showNotif = showNotif;
    function showModal(ct, around) {
        const popup = document.body.appendChild(document.createElement("dialog"));
        popup.appendChild(ct);
        let lastPointerDown;
        popup.addEventListener("pointerdown", function (ev) {
            lastPointerDown = ev.target;
        });
        popup.addEventListener("click", function (ev) {
            if (ev.target instanceof HTMLDialogElement && ev.target === lastPointerDown)
                ev.target.close();
        });
        showDialogModal(popup);
        if (around)
            moveAround(popup, around);
        return popup;
    }
    DIALOG.showModal = showModal;
    function showPopup(ct, around) {
        const popup = document.body.appendChild(document.createElement("dialog"));
        popup.classList.add("popup");
        popup.appendChild(ct);
        popup.addEventListener("pointerdown", function (ev) {
            if (ev.target === this)
                this.close();
        });
        popup.addEventListener("close", function (ev) {
            this.remove();
        });
        popup.showModal();
        moveAround(popup, around);
        return popup;
    }
    DIALOG.showPopup = showPopup;
    function closeAllModals() {
        for (let i = stackModal.length - 1; i >= 0; i--)
            stackModal[i].close();
    }
    DIALOG.closeAllModals = closeAllModals;
    function findDialog(from) {
        let n = from;
        while (n) {
            if (n instanceof HTMLDialogElement)
                return n;
            n = n.parentElement || (n.parentNode instanceof ShadowRoot ? n.parentNode.host : null);
        }
        return null;
    }
    DIALOG.findDialog = findDialog;
    const stackModal = [];
    function showDialogModal(d) {
        d.addEventListener('close', function () {
            const offset = stackModal.indexOf(this);
            for (let i = stackModal.length - 1; i > offset; i--)
                stackModal[i].close();
            stackModal.splice(offset, 1);
            this.remove();
        });
        d.showModal();
        stackModal.push(d);
        return d;
    }
    function moveAround(popup, around) {
        const aroundR = around.getBoundingClientRect();
        const popupR = popup.getBoundingClientRect();
        let firstMove;
        if (aroundR.left > popupR.right) {
            popup.style.marginRight = Math.floor(document.documentElement.clientWidth - aroundR.left) + 'px';
            popup.style.maxWidth = `calc(98vw - ${popup.style.marginRight})`;
            firstMove = 'x';
        }
        else if (aroundR.right < popupR.left) {
            popup.style.marginLeft = Math.ceil(aroundR.right) + 'px';
            popup.style.maxWidth = `calc(98vw - ${popup.style.marginLeft})`;
            firstMove = 'x';
        }
        else if (aroundR.top > popupR.bottom) {
            popup.style.marginBottom = Math.floor(document.documentElement.clientHeight - aroundR.top) + 'px';
            popup.style.maxHeight = `calc(98vh - ${popup.style.marginBottom})`;
            firstMove = 'y';
        }
        else if (aroundR.bottom < popupR.top) {
            popup.style.marginTop = Math.ceil(aroundR.bottom) + 'px';
            popup.style.maxHeight = `calc(98vh - ${popup.style.marginTop})`;
            firstMove = 'y';
        }
        if (firstMove === 'x') {
            if (aroundR.top > popupR.bottom) {
                popup.style.marginBottom = Math.floor(document.documentElement.clientHeight - aroundR.top) + 'px';
                popup.style.maxHeight = `calc(98vh - ${popup.style.marginBottom})`;
            }
            else if (aroundR.bottom < popupR.top) {
                popup.style.marginTop = Math.ceil(aroundR.bottom) + 'px';
                popup.style.maxHeight = `calc(98vh - ${popup.style.marginTop})`;
            }
        }
        else if (firstMove === 'y') {
            if (aroundR.left > popupR.right) {
                popup.style.marginRight = Math.floor(document.documentElement.clientWidth - aroundR.left) + 'px';
                popup.style.maxWidth = `calc(98vw - ${popup.style.marginLeft})`;
            }
            else if (aroundR.right < popupR.left) {
                popup.style.marginLeft = Math.ceil(aroundR.right) + 'px';
                popup.style.maxWidth = `calc(98vw - ${popup.style.marginLeft})`;
            }
        }
        else {
        }
    }
})(DIALOG || (DIALOG = {}));
