import { DB_resData, doCommit, doRequest } from "./backendFacade.js";
import { ScormAPIBase } from "./scorm.js";
export class LocalScormAPI extends ScormAPIBase {
    constructor() {
        super(...arguments);
        this.dirty = false;
    }
    async internalInit(db, resId, options) {
        this.db = db;
        this.scoKey = resId;
        this.initScorm(options);
        this.fields = (await doRequest(db.transaction(DB_resData).objectStore(DB_resData).get(this.scoKey))) || {};
    }
    getLocation() {
        return this.fields.location;
    }
    setLocation(v) {
        if (this.fields.location != v) {
            this.fields.location = v;
            this.dirty = true;
        }
    }
    getStatus() {
        return this.fields.status;
    }
    setStatus(v) {
        if (this.fields.status != v) {
            this.fields.status = v;
            this.dirty = true;
        }
    }
    getScoreRaw() {
        return this.fields.scoreRaw;
    }
    setScoreRaw(v) {
        if (this.fields.scoreRaw != v) {
            this.fields.scoreRaw = v;
            this.dirty = true;
        }
    }
    getScoreMin() {
        return this.fields.scoreMin;
    }
    setScoreMin(v) {
        if (this.fields.scoreMin != v) {
            this.fields.scoreMin = v;
            this.dirty = true;
        }
    }
    getScoreMax() {
        return this.fields.scoreMax;
    }
    setScoreMax(v) {
        if (this.fields.scoreMax != v) {
            this.fields.scoreMax = v;
            this.dirty = true;
        }
    }
    getScoreScaled() {
        return this.fields.scoreScaled;
    }
    setScoreScaled(v) {
        if (this.fields.scoreScaled != v) {
            this.fields.scoreScaled = v;
            this.dirty = true;
        }
    }
    getTotalTime() {
        return this.serializeMsDuration(this.fields.time);
    }
    setSessionTimeOnTerminate(v) {
        if (!(typeof this.fields.time === "number"))
            this.fields.time = 0;
        this.fields.time += this.parseDurationInMs(v);
        this.dirty = true;
        return "";
    }
    getExit() {
        return this.fields.exit;
    }
    setExit(v) {
        if (this.fields.exit != v) {
            this.fields.exit = v;
            this.dirty = true;
        }
    }
    getOther(key, stack) {
        let field = this.fields;
        for (let i = 0; i < stack.length; i++) {
            field = field[stack[i]];
            if (field == null)
                return null;
        }
        return field;
    }
    setOther(key, value) {
        let field = this.fields;
        const stack = key.split('.');
        const imax = stack.length - 1;
        for (let i = 0; i < imax; i++) {
            const f = field[stack[i]];
            if (f == null)
                field = field[stack[i]] = {};
            else
                field = f;
        }
        if (!(stack[imax] in field) || field[stack[imax]] != value) {
            field[stack[imax]] = value;
            this.dirty = true;
        }
        return true;
    }
    doCountFields(stack) {
        let field = this.fields;
        for (let i = 0; i < stack.length - 1; i++) {
            field = field[stack[i]];
            if (field == null)
                return 0;
        }
        let count = 0;
        for (const vKey in field)
            count++;
        return count;
    }
    async doCommit() {
        if (this.dirty) {
            const tr = this.db.transaction(DB_resData, 'readwrite');
            const resRecord = {
                resId: this.scoKey,
                value: this.fields
            };
            tr.objectStore(DB_resData).put(resRecord);
            this.dirty = false;
            return doCommit(tr);
        }
    }
}
