import { DIALOG } from "./dialog.js";
import { JSX, ShadowJsx } from "./jsx.js";
import { makeButton, styleLinkTag } from "./commons.js";
export class RendScorm extends HTMLElement {
    constructor() {
        super();
        this.rendDone = false;
        makeButton(this);
    }
    async onClick(ev) {
        const jsLib = this.getAttribute("js-lib");
        if (jsLib)
            await import(new URL(jsLib, import.meta.url).href);
        const viewer = this.getElementsByTagName("template")[0].content.firstElementChild.cloneNode(true);
        if (this.rendDone) {
            const title = this.textContent;
            if (!await DIALOG.confirm(`Votre rendu est validé, les modifications apportées à '${title}' ne seront pas enregistrées.`, "Ouvrir, j'ai compris", "Abandonner"))
                return;
            viewer.scoOptions = { noStore: true, credit: "no-credit", mode: "review" };
        }
        else if (this.forLearner) {
            viewer.scoOptions = { noStore: true, credit: "no-credit", mode: "review", forLearner: this.forLearner };
        }
        else {
            viewer.scoOptions = null;
        }
        DIALOG.showModal(JSX.createElement("div", { style: "width:90vw; height:90vh; display:flex; flex-direction:column; padding:0;" },
            JSX.createElement(ShadowJsx, null,
                await styleLinkTag("dialog.css"),
                JSX.createElement("header", { style: "display:flex; align-items:center;" },
                    JSX.createElement("h3", { style: "flex:1; margin:.5rem;" },
                        this.dialogTile || this.textContent,
                        this.rendDone ? " (aucun enregistrement)" : ""),
                    JSX.createElement("button", { class: "close", title: "Fermer", onclick: this.onClose })),
                JSX.createElement("main", { style: "display:flex;flex-direction:column; flex:1" }, viewer))));
        viewer.start(this.getAttribute("href"));
    }
    onClose(ev) {
        DIALOG.findDialog(this).close();
    }
}
customElements.define('rend-scorm', RendScorm);
