import { JSX } from "./jsx.js";
import { authenticating, backend, currentUserMode, isTimeReached, openPage, setCurrentRendDialog, setRendState, shadowRoots, timedData } from "./webapp.js";
import { DIALOG } from "./dialog.js";
import { currentTab } from "./tabs.js";
import { showNotNowDialog } from "./widgets.js";
import { styleLinkTag } from "./commons.js";
import { RendScorm } from "./rend-scorm.js";
export class RendArea extends HTMLElement {
    constructor() {
        super(...arguments);
        this.fieldsDirty = new Set();
    }
    setRendValidated(r) {
        this.rendValidated = r;
        const limitReached = this.isLimitReached();
        const readOnly = this.synthUserRend != null;
        if (this.validateBtn) {
            this.validateBtn.disabled = readOnly;
            if (r) {
                if (this.localStoreCheck != null || backend.rend.canRemoteRevoke(this.rendSchema)) {
                    this.validateBtn.textContent = "Reprendre mon rendu";
                    this.validateBtn.hidden = false;
                }
                else {
                    this.validateBtn.hidden = true;
                }
            }
            else {
                this.validateBtn.textContent = "Valider mon rendu";
                this.validateBtn.hidden = false;
            }
        }
        this.setFormReadOnly(r != null || readOnly);
        setRendState(this.rendDt, r, limitReached);
    }
    initForUserRend(data) {
        this.asLearner = true;
        this.synthUserRend = data;
    }
    isLimitReached() {
        var _a;
        return ((_a = this.rend) === null || _a === void 0 ? void 0 : _a.limit) ? isTimeReached(this.rend.limit) : false;
    }
    getRendFieldById(fieldId) {
        var _a;
        return (_a = this.rendSchema.fields) === null || _a === void 0 ? void 0 : _a.find((f) => f.fieldId === fieldId);
    }
    async connectedCallback() {
        var _a, _b, _c;
        if (backend.rendSynthesis.isSynthRendAvailable() && currentUserMode.asCorrector && !this.asLearner) {
            const synth = new (await import("./rendSynth.js")).RendSynth();
            synth.setAttribute("href", this.getAttribute("href"));
            this.replaceWith(synth);
            return;
        }
        if (await initRendUi(this, "rend.css") === false)
            return;
        if (!this.synthUserRend && this.rend.vis && !isTimeReached(this.rend.vis)) {
            this.tryClose();
            showNotNowDialog(this.rend);
            return;
        }
        shadowRoots.add(this.shadowRoot);
        if (this.synthUserRend) {
            this.rendDt.insertAdjacentElement("afterend", JSX.createElement("span", { class: "userRend" },
                "Rendu de ",
                JSX.createElement("span", { class: "user" }, this.synthUserRend.user.pseudo)));
            this.shadowRoot.querySelectorAll("p-score").forEach((tag) => tag.setUserScore(this.synthUserRend));
        }
        else {
            this.shadowRoot.querySelectorAll("p-score").forEach((tag) => tag.refreshScore(backend.scores.fetchedScores));
        }
        const bottomBar = this.shadowRoot.appendChild(JSX.createElement("div", { class: "bottomBar" }));
        const hasStoredFields = this.form && ((_a = this.rendSchema.fields) === null || _a === void 0 ? void 0 : _a.some((f) => f.jsType != null));
        if (this.form)
            this.form.onchange = this.onChangeField;
        let exportBtn;
        if (hasStoredFields && this.rendSchema.fields.every((f) => f.jsType != null)) {
            const exportBtn = JSX.createElement("button", { id: "export" }, this.synthUserRend ? "Télécharger le rendu" : "Télécharger mon rendu");
            exportBtn.addEventListener("click", this.onExport);
            bottomBar.insertAdjacentElement("afterbegin", exportBtn);
        }
        if (this.isLimitReached()) {
            this.rendDt.insertAdjacentElement("afterend", JSX.createElement("span", { class: "msg" }, "Date limite du rendu dépassé"));
        }
        if (!this.synthUserRend) {
            const localStore = hasStoredFields ? await backend.rend.getLocalStoreInfo(this.rendSchema) : null;
            if (localStore != null) {
                this.localStoreCheck = JSX.createElement("input", { id: "lsInput", type: "checkbox" });
                this.localStoreCheck.checked = localStore.lsState;
                this.localStoreCheck.onchange = async () => {
                    this.localStoreCheck.checked = localStore.onLsOptionChange(this.localStoreCheck.checked);
                    if (this.localStoreCheck.checked) {
                        for (const field of this.fieldsDirty)
                            this.saveField(field);
                    }
                    else if (await backend.rend.hasLocalData() && await DIALOG.confirm("Voulez-vous supprimer tous les rendus et leurs validations actuellement enregistrés localement ?", "Tout supprimer", "Préserver les données actuelles")) {
                        await backend.rend.clearLocalData();
                    }
                };
                this.form.insertAdjacentElement("afterend", JSX.createElement("label", { id: "lsLabel" },
                    this.localStoreCheck,
                    JSX.createElement("span", null, localStore.lsOptionLabel)));
            }
            else if (!backend.rend.canRemoteSaveDraft(this.rendSchema) && !backend.rend.canRemoteValidate(this.rendSchema)) {
                bottomBar.insertAdjacentElement("afterbegin", backend.rend.buildMsgNoStorageAvailable(JSX.createElement("span", { class: "msg" })));
            }
            if (backend.rend.canRemoteValidate(this.rendSchema) || localStore != null) {
                this.validateBtn = JSX.createElement("button", { id: "validate" });
                this.validateBtn.onclick = this.onValidate;
                bottomBar.insertAdjacentElement("afterbegin", this.validateBtn);
            }
        }
        this.setFormReadOnly(true);
        if (exportBtn)
            exportBtn.disabled = true;
        if (this.form) {
            try {
                const fields = this.synthUserRend || await backend.rend.fetchLastFields(this.rendSchema);
                if (fields) {
                    const tw = document.createTreeWalker(this.form, NodeFilter.SHOW_ELEMENT);
                    while (tw.nextNode()) {
                        const n = tw.currentNode;
                        if (n instanceof HTMLTextAreaElement) {
                            n.value = ((_b = fields.getField(n.name)) === null || _b === void 0 ? void 0 : _b.getString()) || "";
                        }
                        else if (n instanceof HTMLInputElement) {
                            if (n.type === "url") {
                                n.value = ((_c = fields.getField(n.name)) === null || _c === void 0 ? void 0 : _c.getString()) || "";
                                const wrapper = JSX.createElement("div", { class: "inputCtn" },
                                    JSX.createElement("button", { class: "gotoUrl", title: "Ouvrir cette url", onclick: this.onGotoUrl }));
                                n.replaceWith(wrapper);
                                wrapper.insertAdjacentElement("afterbegin", n);
                                this.onInputUrlChange.call(n);
                                n.addEventListener('input', this.onInputUrlChange);
                            }
                            else if (n.type === "file") {
                                const field = fields.getField(n.name);
                                if (field && field.fileName) {
                                    const transfer = new DataTransfer();
                                    transfer.items.add(new File([], field.fileName));
                                    n.files = transfer.files;
                                    n.blobField = field;
                                }
                                else {
                                    n.value = "";
                                }
                                const wrapper = JSX.createElement("div", { class: "inputCtn" },
                                    JSX.createElement("button", { class: "downloadFile", title: "Télécharger le fichier", onclick: this.onDownloadFile }),
                                    JSX.createElement("button", { class: "deleteFile", title: "Supprimer le fichier", onclick: this.onDeleteFile }));
                                n.replaceWith(wrapper);
                                wrapper.insertAdjacentElement("afterbegin", n);
                                this.onInputFileChange.call(n);
                                n.addEventListener('change', this.onInputFileChange);
                            }
                        }
                    }
                }
                this.setRendValidated(this.synthUserRend ? this.synthUserRend.valid : backend.rend.isValidated(this.rend.id));
                if (exportBtn)
                    exportBtn.disabled = false;
            }
            catch (e) {
                this.onError(e);
            }
        }
    }
    disconnectedCallback() {
        shadowRoots.delete(this.shadowRoot);
    }
    setFormReadOnly(readOnly) {
        if (!this.form)
            return;
        const tw = document.createTreeWalker(this.form, NodeFilter.SHOW_ELEMENT);
        this.form.ariaDisabled = readOnly ? "true" : "false";
        while (tw.nextNode()) {
            const n = tw.currentNode;
            if (n instanceof HTMLTextAreaElement || n instanceof HTMLInputElement) {
                n.disabled = readOnly;
                if (n.type === "file")
                    this.onInputFileChange.call(n);
            }
            else if (n instanceof RendScorm) {
                n.rendDone = this.rendValidated != null;
            }
        }
    }
    onChangeField(ev) {
        const rendArea = this.getRootNode().host;
        rendArea.saveField(ev.target);
    }
    onExport(ev) {
        this.getRootNode().host.doExport();
    }
    onValidate(ev) {
        this.getRootNode().host.doValidate();
    }
    onDeleteFile(ev) {
        var _a;
        ev.preventDefault();
        ev.stopImmediatePropagation();
        const rendArea = this.getRootNode().host;
        const input = this.parentElement.getElementsByTagName("input")[0];
        if (((_a = input.files) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            input.blobField = undefined;
            input.value = "";
            rendArea.saveField(input);
            rendArea.onInputFileChange.call(input);
        }
    }
    async onDownloadFile(ev) {
        ev.preventDefault();
        ev.stopImmediatePropagation();
        const input = this.parentElement.getElementsByTagName("input")[0];
        await fetchRendBlob(input);
        const file = input.files.item(0);
        if (file) {
            const url = window.URL.createObjectURL(file);
            const a = JSX.createElement("a", { hidden: true, href: url, download: file.name, target: "_blank" });
            document.body.appendChild(a);
            a.click();
            setTimeout(() => {
                a.remove();
                window.URL.revokeObjectURL(url);
            }, 0);
        }
    }
    onInputFileChange() {
        const disabled = this.files.item(0) == null;
        const downloadBtn = this.nextElementSibling;
        if (downloadBtn) {
            downloadBtn.disabled = disabled;
            const delBtn = downloadBtn.nextElementSibling;
            if (delBtn)
                delBtn.disabled = disabled || this.disabled;
        }
    }
    onGotoUrl() {
        const input = this.previousElementSibling;
        if (input && checkUrlSyntax(input.value))
            open(input.value, "_blank");
    }
    onInputUrlChange() {
        const btn = this.nextElementSibling;
        if (btn)
            btn.disabled = !checkUrlSyntax(this.value);
    }
    saveField(tag) {
        const save = async (field, fieldKey, value) => {
            var _a;
            if (await backend.rend.onFieldChange(this.rendSchema, field, fieldKey, value, (_a = this.localStoreCheck) === null || _a === void 0 ? void 0 : _a.checked)) {
                this.fieldsDirty.delete(tag);
            }
            else {
                this.fieldsDirty.add(tag);
            }
        };
        if (tag instanceof HTMLTextAreaElement) {
            save(this.getRendFieldById(tag.name), tag.name, tag.value);
        }
        else if (tag instanceof HTMLInputElement) {
            if (tag.type === "url")
                save(this.getRendFieldById(tag.name), tag.name, tag.value);
            if (tag.type === "file") {
                const file = tag.files.item(0);
                save(this.getRendFieldById(tag.name), tag.name, file);
            }
        }
    }
    async tryClose() {
        const dialog = this.parentElement;
        if (this.fieldsDirty.size > 0) {
            if (await DIALOG.confirm("Vos modifications vont être perdues, souhaitez-vous continuer ?", "Fermer et abandonner ma saisie", "Retourner dans mon rendu") == false)
                return;
        }
        setCurrentRendDialog(null);
        dialog.close();
        dialog.remove();
    }
    async doExport() {
        const [vars, skin] = await Promise.all([(await fetch(`${window.skinRoot}skin.css`)).text(), (await fetch(`${window.skinRoot}exportedRend.css`)).text()]);
        for (const input of this.form.elements) {
            if (input instanceof HTMLInputElement && input.type === "file")
                await fetchRendBlob(input);
        }
        const user = backend.user;
        const isDisabled = this.form.ariaDisabled === "true";
        if (isDisabled)
            this.setFormReadOnly(false);
        const fields = new FormData(this.form);
        if (isDisabled)
            this.setFormReadOnly(false);
        const doc = new DOMParser().parseFromString(this.shadowRoot.getElementById("rendExport").getAttribute("p-export"), "text/xml");
        const tw = doc.createTreeWalker(doc.documentElement, Node.ELEMENT_NODE);
        const waiters = [];
        for (;;) {
            const elt = tw.currentNode;
            const next = tw.nextNode();
            if (elt.localName.startsWith("inject-")) {
                const keyField = elt.getAttribute("field");
                switch (elt.localName) {
                    case "inject-skin":
                        const style = doc.createElementNS(XHTMLNS, "style");
                        style.append(vars, skin);
                        elt.insertAdjacentElement("beforebegin", style);
                        break;
                    case "inject-userinfo":
                        if (user === null || user === void 0 ? void 0 : user.pseudo) {
                            const div = doc.createElementNS(XHTMLNS, "div");
                            div.setAttribute("class", "user");
                            div.append(user.pseudo);
                            elt.insertAdjacentElement("beforebegin", div);
                        }
                        break;
                    case "inject-field-text":
                        const divTxt = doc.createElementNS(XHTMLNS, "div");
                        divTxt.setAttribute("class", "field");
                        divTxt.append(fields.get(keyField) || "");
                        elt.insertAdjacentElement("beforebegin", divTxt);
                        break;
                    case "inject-field-url":
                        const tagUrl = doc.createElementNS(XHTMLNS, "a");
                        tagUrl.setAttribute("class", "field");
                        tagUrl.setAttribute("target", "_blank");
                        const url = fields.get(keyField) || "";
                        if (url && url.startsWith("http"))
                            tagUrl.setAttribute("href", url);
                        tagUrl.append(url);
                        elt.insertAdjacentElement("beforebegin", tagUrl);
                        break;
                    case "inject-field-file":
                        const tagFile = doc.createElementNS(XHTMLNS, "a");
                        tagFile.setAttribute("class", "field");
                        tagFile.setAttribute("target", "_blank");
                        const file = fields.get(keyField);
                        if ((file === null || file === void 0 ? void 0 : file.size) > 0) {
                            tagFile.setAttribute("download", file.name);
                            const reader = new FileReader();
                            let done;
                            waiters.push(new Promise((rslv) => {
                                done = rslv;
                            }));
                            reader.addEventListener("load", () => {
                                tagFile.setAttribute("href", reader.result);
                                done();
                            });
                            reader.readAsDataURL(file);
                            tagFile.append(file.name);
                        }
                        else
                            tagFile.append(file ? "Fichier vide" : "Aucun fichier sélectionné");
                        elt.insertAdjacentElement("beforebegin", tagFile);
                        break;
                }
                elt.remove();
            }
            if (!next)
                break;
        }
        await Promise.all(waiters);
        const a = document.createElement('a');
        a.href = URL.createObjectURL(new Blob([new XMLSerializer().serializeToString(doc)], { type: 'text/html' }));
        a.download = `rendu.html`;
        a.click();
    }
    async doValidate() {
        try {
            if (this.rendValidated) {
                if (this.localStoreCheck != null || backend.rend.canRemoteRevoke(this.rendSchema)) {
                    await backend.rend.revoke(this.rendSchema);
                }
            }
            else {
                await backend.rend.validate(new FormData(this.form), this.rendSchema);
            }
            this.fieldsDirty.clear();
        }
        catch (e) {
            this.onError(e);
        }
        this.setRendValidated(backend.rend.isValidated(this.rendSchema.rendId));
    }
    onError(e) {
        if (e === "rendLocked") {
            DIALOG.showNotif("La date limite pour ce rendu a été atteinte, aucune modification n'est à présent possible.");
        }
        else if (e === "alreadyValidted") {
            DIALOG.showNotif("Ce rendu a été validé et ne peut être modifié.");
        }
        else {
            DIALOG.showNotif("Une erreur s'est produite. Recommencez ultérieurement.");
            console.log(e);
        }
    }
}
const XHTMLNS = 'http://www.w3.org/1999/xhtml';
customElements.define('p-rendarea', RendArea);
export async function initRendUi(tag, cssUrl) {
    var _a;
    if (authenticating) {
        tag.tryClose();
        return false;
    }
    const sr = tag.attachShadow({ mode: "open" });
    const closeBtn = JSX.createElement("button", { id: "close", title: "Fermer" });
    closeBtn.addEventListener("click", function () {
        this.getRootNode().host.tryClose();
    });
    sr.append(await styleLinkTag(cssUrl), closeBtn);
    const resp = await fetch(tag.getAttribute("href"));
    if (resp.ok) {
        closeBtn.insertAdjacentHTML("afterend", await resp.text());
        tag.form = sr.getElementById("rendForm");
        tag.rendDt = sr.querySelector("[p-rendid]");
        const rendId = tag.rendDt.getAttribute("p-rendid");
        tag.rend = (_a = timedData(rendId)) === null || _a === void 0 ? void 0 : _a.rend;
        tag.rendSchema = tag.form ? JSON.parse(tag.form.getAttribute("p-scheme")) : { rendId };
    }
    if (!tag.rend) {
        tag.tryClose();
        return false;
    }
    const showActBtn = sr.getElementById("showAct");
    if (showActBtn) {
        const href = showActBtn.getAttribute("href");
        showActBtn.removeAttribute("href");
        if (href.indexOf('#') < 0 && (currentTab === null || currentTab === void 0 ? void 0 : currentTab.matchHref(href))) {
            showActBtn.hidden = true;
        }
        else {
            showActBtn.title = "Afficher en arrière plan";
            showActBtn.onclick = function () {
                openPage(href);
                this.hidden = true;
            };
        }
    }
    setCurrentRendDialog(tag);
    return sr;
}
export async function fetchRendBlob(input) {
    const file = input.files.item(0);
    if ((file === null || file === void 0 ? void 0 : file.size) === 0 && input.blobField) {
        const blob = await input.blobField.getBlob();
        if (blob) {
            const transfer = new DataTransfer();
            transfer.items.add(new File([blob], input.blobField.fileName));
            input.files = transfer.files;
        }
    }
}
export function checkUrlSyntax(url) {
    if (url.startsWith("http://") || url.startsWith("https://"))
        try {
            new URL(url);
            return true;
        }
        catch (e) { }
    return false;
}
