import { backend, isTimeReached, setCurrentRendDialog, shadowRoots } from "./webapp.js";
import { JSX, ShadowJsx } from "./jsx.js";
import { checkUrlSyntax, initRendUi, RendArea } from "./rend.js";
import { DIALOG } from "./dialog.js";
import { SynthUserImpl } from "./backendFacade.js";
import { makeButton, styleLinkTag } from "./commons.js";
import { ScoreSticker } from "./widgets.js";
export class RendSynth extends HTMLElement {
    constructor() {
        super(...arguments);
        this.cohort = null;
    }
    async connectedCallback() {
        var _a;
        if ((await initRendUi(this, "rendSynth.css")) === false)
            return;
        this.synthRend = backend.rendSynthesis.canSynthRend(this.rendSchema);
        if (!this.synthRend.valid && (!this.synthRend.fields || this.synthRend.fields.length === 0)) {
            const rend = new RendArea();
            rend.asLearner = true;
            rend.setAttribute("href", this.getAttribute("href"));
            this.replaceWith(rend);
            return;
        }
        if (this.rend.limit && !isTimeReached(this.rend.limit)) {
        }
        this.rendDt.insertAdjacentElement("afterend", JSX.createElement("button", { id: "showRend", onclick: this.onShowRend }, "Visualiser"));
        (_a = this.shadowRoot.getElementById("rendExport")) === null || _a === void 0 ? void 0 : _a.remove();
        const scoreIds = backend.scores.isScoresInSynthAvailable() ? [] : null;
        this.allColumns = [new RendValidColumn(this.rend.limit ? new Date(this.rend.limit).getTime() : undefined)];
        for (const tag of this.shadowRoot.querySelectorAll(".fieldLbl,p-score")) {
            if (tag instanceof ScoreSticker) {
                this.allColumns.push(new RendScoreColumn(tag));
                scoreIds === null || scoreIds === void 0 ? void 0 : scoreIds.push(tag.getAttribute("score-id"));
            }
            else {
                const fieldId = tag.getAttribute("field-id");
                if (fieldId) {
                    const field = this.synthRend.fields.find((f) => f.fieldId === fieldId);
                    if (field)
                        switch (field.jsType) {
                            case "string":
                                this.allColumns.push(new RendFieldStringColumn(field, tag));
                                break;
                            case "File":
                                this.allColumns.push(new RendFieldFileColumn(field, tag));
                                break;
                            case null:
                                this.allColumns.push(new RendFieldScormColumn(field, tag));
                                break;
                            default:
                                console.log("field.jsType unknown:::", field);
                        }
                }
            }
        }
        const body = this.shadowRoot.getElementById("body");
        const waiting = JSX.createElement("div", { class: "msg" }, "Chargement des données...");
        body.replaceWith(waiting);
        this.cohort = await backend.fetchCohort();
        this.userData = this.cohort.users.map((u) => new SynthUserImpl(this.rendSchema, u)).sort((u1, u2) => u1.user.pseudo.localeCompare(u2.user.pseudo));
        this.columns = [this.allColumns[0]];
        const fields = [];
        let widthFree = window.innerWidth - 300;
        for (let i = 1; i < this.allColumns.length && widthFree > 0; i++) {
            const column = this.allColumns[i];
            widthFree -= column.width;
            if (column instanceof RendFieldColumn)
                fields.push(column.field);
            this.columns.push(column);
        }
        if ((scoreIds === null || scoreIds === void 0 ? void 0 : scoreIds.length) > 0) {
            await backend.scores.fetchScoresFor(scoreIds, this.userData);
        }
        await backend.rendSynthesis.fetchSynthRend(this.rendSchema, fields, this.userData);
        this.table = JSX.createElement("table", null,
            JSX.createElement("tr", { class: "head" },
                JSX.createElement("th", null,
                    JSX.createElement("div", { class: "headCol" },
                        JSX.createElement("span", null, backend.isSendMsgAvailable() ? JSX.createElement("button", { class: "sendMsg", id: "sendMsgToCohort", onclick: this.onSenMsgToCohort, title: "Envoyer un message à plusieurs destinataires" }) : undefined),
                        JSX.createElement("synthrend-editcol", null))),
                this.columns.map((c) => c.fillHeader(JSX.createElement("th", null)))),
            this.userData.map(u => JSX.createElement("tr", null,
                JSX.createElement("th", null,
                    JSX.createElement("span", null,
                        JSX.createElement("button", { class: "user", onclick: () => { this.onShowRendFor(u); } }, u.user.pseudo),
                        backend.isSendMsgAvailable() ? JSX.createElement("button", { class: "sendMsg", onclick: () => { this.onSenMsg(u); }, title: "Envoyer un message" }) : undefined)),
                this.columns.map((c) => c.fillCell(JSX.createElement("td", null), u)))));
        waiting.replaceWith(this.table);
        shadowRoots.add(this.shadowRoot);
    }
    disconnectedCallback() {
        shadowRoots.delete(this.shadowRoot);
    }
    onShowRend() {
        const rend = new RendArea();
        rend.asLearner = true;
        rend.setAttribute("href", this.getRootNode().host.getAttribute("href"));
        DIALOG.showModal(rend);
    }
    async onShowRendFor(u) {
        const rend = new RendArea();
        await backend.rendSynthesis.fetchSynthRendAllFields(this.rendSchema, u);
        rend.initForUserRend(u);
        rend.setAttribute("href", this.getAttribute("href"));
        DIALOG.showModal(rend);
    }
    async onSenMsgToCohort() {
        const me = this.getRootNode().host;
        const validated = me.userData.filter((userRend) => userRend.valid != null);
        async function send(users) {
            backend.sendMsgTo({ users: users.map(userRend => userRend.user) }, `Rendu : ${me.rend.act.ti}`);
        }
        DIALOG.showPopup(JSX.createElement("div", null,
            JSX.createElement(ShadowJsx, null,
                await styleLinkTag("menu.css"),
                JSX.createElement("menu", null,
                    JSX.createElement("li", null,
                        JSX.createElement("button", { class: "menuEntry", onclick: () => {
                                send(me.userData);
                            } }, `Envoyer un message à tous (${me.userData.length})`)),
                    validated.length > 0 && validated.length < me.userData.length ? [
                        JSX.createElement("li", null,
                            JSX.createElement("button", { class: "menuEntry", onclick: () => { send(me.userData.filter((userRend) => userRend.valid == null)); } }, `Envoyer un message à ceux dont les rendus ne sont pas encore validés (${me.userData.length - validated.length})`)),
                        JSX.createElement("li", null,
                            JSX.createElement("button", { class: "menuEntry", onclick: () => { send(validated); } }, `Envoyer un message à ceux dont les rendus sont validés (${validated.length})`))
                    ] : undefined))), this);
    }
    onSenMsg(u) {
        backend.sendMsgTo({ users: [u.user] }, `Rendu : ${this.rend.act.ti}`);
    }
    tryClose() {
        const dialog = this.parentElement;
        setCurrentRendDialog(null);
        dialog.close();
        dialog.remove();
    }
}
customElements.define('p-rendsynth', RendSynth);
class RendColumn {
    get width() { return 150; }
    fillHeader(cell) {
        this.cell = cell;
        cell.rendColumn = this;
        cell.appendChild(JSX.createElement("div", { class: "headCol" },
            this.createLabel(),
            JSX.createElement("synthrend-editcol", null)));
        return cell;
    }
    removeCol() {
        if (!this.cell)
            return;
        let offset = offsetNode(this.cell);
        let tr = this.cell.parentElement.nextElementSibling;
        this.cell.remove();
        while (tr) {
            tr.childNodes.item(offset).remove();
            tr = tr.nextElementSibling;
        }
        this.cell = null;
    }
    insertColAfter(cellHead, rendSynth) {
        this.removeCol();
        let offset = offsetNode(cellHead);
        cellHead.insertAdjacentElement("afterend", this.fillHeader(JSX.createElement("th", null)));
        let row = cellHead.parentElement.nextElementSibling;
        let i = 0;
        while (row) {
            row.childNodes.item(offset).insertAdjacentElement("afterend", this.fillCell(JSX.createElement("td", null), rendSynth.userData[i++]));
            row = row.nextElementSibling;
        }
    }
}
class RendValidColumn extends RendColumn {
    constructor(limit) {
        super();
        this.limit = limit;
    }
    fillCell(cell, userRend) {
        if (!userRend.valid) {
            cell.replaceChildren(JSX.createElement("div", { class: "notValidated" }, "Non validé"));
        }
        else {
            const afterLimit = userRend.valid.ts && this.limit && userRend.valid.ts > this.limit;
            cell.replaceChildren(JSX.createElement("div", { class: afterLimit ? "validated afterLimit" : "validated", title: afterLimit ? "Rendu après la date limite !" : undefined }, userRend.valid.ts ? new Date(userRend.valid.ts).toLocaleString() : "Oui"));
        }
        return cell;
    }
    createLabel() {
        return JSX.createElement("span", null, "Rendu validé");
    }
}
class RendFieldColumn extends RendColumn {
    constructor(field, label) {
        super();
        this.field = field;
        this.label = label;
    }
    createLabel() {
        return cloneContent(this.label);
    }
}
function cloneContent(ctn) {
    const sp = JSX.createElement("span", null);
    for (let ch = ctn.firstChild; ch; ch = ch.nextSibling) {
        sp.appendChild(ch.cloneNode(true));
    }
    return sp;
}
class RendFieldStringColumn extends RendFieldColumn {
    get width() { return 300; }
    fillCell(cell, userRend) {
        var _a;
        const content = (_a = userRend.getField(this.field.fieldId)) === null || _a === void 0 ? void 0 : _a.getString();
        if (!content) {
            cell.replaceChildren(JSX.createElement("div", { class: "notFilled" }));
        }
        else {
            const input = this.label.nextElementSibling;
            let type = "";
            if (input instanceof HTMLInputElement) {
                type = input.type;
                switch (type) {
                    case "url":
                        cell.replaceChildren(JSX.createElement("div", { class: "filled" },
                            JSX.createElement("a", { href: checkUrlSyntax(content) ? content : undefined, target: "_blank" }, content)));
                        return cell;
                }
            }
            else {
                type = "textarea";
            }
            cell.replaceChildren(JSX.createElement("div", { class: `filled ${type}` }, content));
        }
        return cell;
    }
}
class RendFieldFileColumn extends RendFieldColumn {
    fillCell(cell, userRend) {
        var _a;
        const fileName = (_a = userRend.getField(this.field.fieldId)) === null || _a === void 0 ? void 0 : _a.fileName;
        if (!fileName) {
            cell.replaceChildren(JSX.createElement("div", { class: "notFilled" }));
        }
        else {
            cell.replaceChildren(JSX.createElement("div", { class: "filled" },
                JSX.createElement("button", { onclick: () => this.onClick(userRend) }, fileName)));
        }
        return cell;
    }
    async onClick(userRend) {
        const field = userRend.getField(this.field.fieldId);
        if (!field)
            return;
        const blob = await field.getBlob();
        if (blob) {
            const url = window.URL.createObjectURL(blob);
            const a = JSX.createElement("a", { hidden: true, href: url, download: field.fileName, target: "_blank" });
            document.body.appendChild(a);
            a.click();
            setTimeout(() => {
                a.remove();
                window.URL.revokeObjectURL(url);
            }, 0);
        }
    }
}
class RendFieldScormColumn extends RendFieldColumn {
    createLabel() {
        return JSX.createElement("span", null, this.label.textContent);
    }
    fillCell(cell, userRend) {
        const rendScorm = this.label.cloneNode(true);
        rendScorm.forLearner = userRend.user;
        rendScorm.dialogTile = rendScorm.lastChild.nodeValue;
        rendScorm.lastChild.nodeValue = "Ouvrir";
        cell.replaceChildren(JSX.createElement("div", { class: "center" }, rendScorm));
        return cell;
    }
}
class RendScoreColumn extends RendColumn {
    constructor(template) {
        super();
        this.template = template;
    }
    createLabel() {
        let fieldLabel;
        for (const tag of this.template.parentElement.parentElement.children) {
            if (tag.classList.contains("fieldLbl")) {
                fieldLabel = tag;
                break;
            }
        }
        const scoreLabel = this.template.getAttribute("label");
        return JSX.createElement("span", null,
            JSX.createElement("span", { class: "scoreOf" }, fieldLabel ? cloneContent(fieldLabel) : "Rendu"),
            JSX.createElement("span", { class: "scoreLabel" }, scoreLabel ? `, note : ${scoreLabel}` : " : note"));
    }
    fillCell(cell, userScores) {
        const sticker = this.template.cloneNode(true);
        sticker.removeAttribute("label");
        sticker.setUserScore(userScores);
        cell.replaceChildren(sticker);
        return cell;
    }
}
class SynthRendEditCol extends HTMLElement {
    constructor() {
        super();
        makeButton(this);
    }
    async onClick() {
        const th = this.closest("th");
        this.trHead = th.parentElement;
        const rendSynth = this.trHead.getRootNode().host;
        this.insertOffset = offsetNode(th);
        const selector = JSX.createElement("div", { class: "selector" },
            JSX.createElement(ShadowJsx, null,
                JSX.createElement("link", { rel: "stylesheet", href: `${window.skinRoot}rendSynthEditCol.css`, blocking: "render" }),
                JSX.createElement("h4", null, "Ajout et suppression de colonnes"),
                rendSynth.allColumns.map((c) => JSX.createElement("label", null,
                    JSX.createElement("input", { type: "checkbox", checked: c.cell != null ? "" : undefined, onclick: (ev) => {
                            const input = ev.target;
                            if (input.checked)
                                this.insertCol(c, rendSynth);
                            else
                                this.removeCol(c);
                        } }),
                    c.createLabel()))));
        DIALOG.showModal(selector);
    }
    async insertCol(newCol, rendSynth) {
        if (newCol instanceof RendFieldColumn)
            await backend.rendSynthesis.fetchSynthRendAddField(rendSynth.rendSchema, newCol.field, rendSynth.userData);
        newCol.insertColAfter(this.trHead.childNodes.item(this.insertOffset++), rendSynth);
    }
    removeCol(col) {
        if (col.cell) {
            if (this.insertOffset >= offsetNode(col.cell))
                this.insertOffset--;
            col.removeCol();
        }
    }
}
customElements.define('synthrend-editcol', SynthRendEditCol);
function offsetNode(elt) {
    let p = elt.previousSibling;
    let offset = 0;
    while (p) {
        p = p.previousSibling;
        offset++;
    }
    return offset;
}
