import { backend, scoreDefsModule, shadowRoots } from "./webapp.js";
import { styleLinkTag } from "./commons.js";
import { JSX, ShadowJsx } from "./jsx.js";
import { isScoreDataComputable, isScoreDefAnnotable, isScoreDefCompletion, isScoreDefValuable } from "./scores.js";
import { DIALOG } from "./dialog.js";
import { TabButton } from "./tab-button.js";
import { escapeCsv } from "./appCommons.js";
export class ScoresSynth extends HTMLElement {
    get scoreId() { return this.getAttribute("score-id"); }
    async connectedCallback() {
        const sr = this.attachShadow({ mode: "open" });
        shadowRoots.add(sr);
        sr.append(await styleLinkTag("scoreSynth.css"));
        const msg = sr.appendChild(JSX.createElement("div", { class: "msg" }, "Chargement des données..."));
        const cohort = await backend.fetchCohort();
        const usersScore = cohort.users.map((u) => new SynthUserScoreSingle(u)).sort((u1, u2) => u1.user.pseudo.localeCompare(u2.user.pseudo));
        const scoreId = this.scoreId;
        await backend.scores.fetchScoresFor([scoreId], usersScore);
        msg.remove();
        const table = JSX.createElement("div", { id: "details", role: "tabpanel", "aria-labelledby": "detailsBtn" },
            JSX.createElement("table", null, usersScore.map(u => {
                const score = JSX.createElement("p-score", { "score-id": scoreId });
                score.setUserScore(u);
                return JSX.createElement("tr", null,
                    JSX.createElement("th", null,
                        JSX.createElement("span", null,
                            JSX.createElement("span", { class: "user" }, u.user.pseudo),
                            backend.isSendMsgAvailable() ? JSX.createElement("button", { class: "sendMsg", onclick: () => { this.onSenMsg(u); }, title: "Envoyer un message" }) : undefined)),
                    JSX.createElement("td", null, score));
            })),
            JSX.createElement("div", { id: "footer" },
                backend.isSendMsgAvailable() ? JSX.createElement("button", { class: "sendMsg", id: "sendMsgToCohort", onclick: () => { this.onSenMsgToCohort(def, usersScore); }, title: "Envoyer un message à plusieurs destinataires" }) : undefined,
                JSX.createElement("span", { style: "flex:1" }),
                JSX.createElement("button", { id: "exportCsv", onclick: () => { this.onExportCsv(def, usersScore); } }, "Exporter en CSV")));
        const mod = await scoreDefsModule();
        const def = mod.getScoreDef(scoreId);
        if (isScoreDefValuable(def)) {
            const stats = JSX.createElement("p-scoresstats", { id: "stats", role: "tabpanel", "aria-labelledby": "statsBtn", hidden: "" });
            stats.def = def;
            stats.usersScore = usersScore;
            sr.append(JSX.createElement("nav", { role: "tablist" },
                JSX.createElement(TabButton, { id: "detailsBtn", role: "tab", "aria-controls": "details", "aria-selected": "true" }, "Détails"),
                JSX.createElement(TabButton, { id: "statsBtn", role: "tab", "aria-controls": "stats", "aria-selected": "false" }, "Statistiques")), table, stats);
        }
        else {
            sr.append(table);
        }
    }
    disconnectedCallback() {
        shadowRoots.delete(this.shadowRoot);
    }
    onSenMsg(u) {
        backend.sendMsgTo({ users: [u.user] });
    }
    async onSenMsgToCohort(def, usersScore) {
        await backend.scores.fetchScoresFor([def.id], usersScore);
        const scored = usersScore.filter((u) => isScoreDataComputable(u.scores.get(def.id)));
        async function send(users) {
            if (users.length === 0) {
                DIALOG.showNotif("Aucun destinataire trouvé avec ces critères");
            }
            else {
                backend.sendMsgTo({ users: users.map(userRend => userRend.user) });
            }
        }
        DIALOG.showPopup(JSX.createElement("div", null,
            JSX.createElement(ShadowJsx, null,
                await styleLinkTag("menu.css"),
                JSX.createElement("menu", null,
                    JSX.createElement("li", null,
                        JSX.createElement("button", { class: "menuEntry", onclick: () => {
                                send(usersScore);
                            } }, `Envoyer un message à tous (${usersScore.length})`)),
                    scored.length > 0 && scored.length < usersScore.length ? [
                        JSX.createElement("li", null,
                            JSX.createElement("button", { class: "menuEntry", onclick: () => { send(usersScore.filter((u) => !isScoreDataComputable(u.scores.get(def.id)))); } }, `Envoyer un message à ceux qui n'ont pas de note (${usersScore.length - scored.length})`)),
                        JSX.createElement("li", null,
                            JSX.createElement("button", { class: "menuEntry", onclick: () => { send(scored); } }, `Envoyer un message à ceux qui ont une note (${scored.length})`))
                    ] : undefined,
                    def.type === "Comput" && scored.length > 0 ? [
                        JSX.createElement("li", null,
                            JSX.createElement("button", { class: "menuEntry", onclick: function () {
                                    const cpl = parseInt(this.nextElementSibling.value) / 100;
                                    send(usersScore.filter((u) => {
                                        var _a;
                                        const score = u.scores.get(def.id);
                                        return !isScoreDataComputable(score) || ((_a = score.c) !== null && _a !== void 0 ? _a : 1) <= cpl;
                                    }));
                                } }, "Envoyer un message à ceux qui ont une complétude inférieure ou égale à "),
                            JSX.createElement("input", { type: "number", step: "1", min: "0", max: "100", value: "50", style: "width:3em;" }),
                            "%"),
                        JSX.createElement("li", null,
                            JSX.createElement("button", { class: "menuEntry", onclick: function () {
                                    const cpl = parseInt(this.nextElementSibling.value) / 100;
                                    send(scored.filter((u) => {
                                        var _a;
                                        const score = u.scores.get(def.id);
                                        return ((_a = score.c) !== null && _a !== void 0 ? _a : 1) >= cpl;
                                    }));
                                } }, "Envoyer un message à ceux qui ont une complétude supérieure ou égale à "),
                            JSX.createElement("input", { type: "number", step: "1", min: "0", max: "100", value: "50", style: "width:3em;" }),
                            "%")
                    ] : undefined))), this);
    }
    async onExportCsv(def, usersScore) {
        await backend.scores.fetchScoresFor([def.id], usersScore);
        const link = document.createElement('a');
        const data = ['"Utilisateur"'];
        const isValuable = isScoreDefValuable(def);
        if (isValuable)
            data.push(`;"Note [${def.min}-${def.max}]"`);
        const isAnnotable = isScoreDefAnnotable(def);
        if (isAnnotable)
            data.push(";", escapeCsv(def.annotLabel));
        data.push("\r\n");
        for (const us of usersScore) {
            data.push(escapeCsv(us.user.pseudo));
            const score = us.scores.get(def.id);
            if (isValuable) {
                data.push(";");
                if (score && isScoreDataComputable(score))
                    data.push(score.s.toString());
            }
            if (isAnnotable) {
                data.push(";");
                if (score && score.a)
                    data.push(escapeCsv(score.a));
            }
            data.push("\r\n");
        }
        const url = URL.createObjectURL(new Blob(data));
        link.href = url;
        link.download = "scores.csv";
        link.click();
        setTimeout(function () { URL.revokeObjectURL(url); }, 0);
    }
}
customElements.define('p-scoressynth', ScoresSynth);
export class SynthUserScoreSingle {
    constructor(user) {
        this.user = user;
    }
}
class ScoreStats extends HTMLElement {
    constructor() {
        super(...arguments);
        this.countUsers = JSX.createElement("span", { class: "val", id: "countUsers" });
        this.totalUsers = JSX.createElement("span", { class: "val", id: "totalUsers" });
        this.scoreMin = JSX.createElement("span", { class: "val", id: "scoreMin" });
        this.scoreMax = JSX.createElement("span", { class: "val", id: "scoreMax" });
        this.scoreMed = JSX.createElement("span", { class: "val", id: "scoreMed" });
        this.scoreAvrg = JSX.createElement("span", { class: "val", id: "scoreAvrg" });
    }
    static get observedAttributes() { return ['hidden']; }
    async attributeChangedCallback(name, oldValue, newValue) {
        if (newValue == null) {
            if (!this.shadowRoot) {
                const sr = this.attachShadow({ mode: "open" });
                sr.append(await styleLinkTag("scoreStats.css"));
                if (isScoreDefCompletion(this.def)) {
                    function titleColumn(prevPercent, currentPercent) {
                        return `Population dont la complétude de la note est comprise entre ${prevPercent} (exclus) et ${currentPercent} (inclus).`;
                    }
                    this.cplUserCells = [];
                    for (let i = 0; i < 12; i++)
                        this.cplUserCells.push(JSX.createElement("td", null));
                    sr.append(JSX.createElement("table", { id: "cplTbl" },
                        JSX.createElement("tr", null,
                            JSX.createElement("th", null, "Complétude"),
                            JSX.createElement("td", { title: "Population dont la complétude de la note est nulle." }, "0%"),
                            JSX.createElement("td", { title: "Population dont la complétude de la note est inférieure ou égale à 10%, mais pas nulle." }, "≤10%"),
                            JSX.createElement("td", { title: titleColumn("10%", "20%") }, "≤20%"),
                            JSX.createElement("td", { title: titleColumn("20%", "30%") }, "≤30%"),
                            JSX.createElement("td", { title: titleColumn("30%", "40%") }, "≤40%"),
                            JSX.createElement("td", { title: titleColumn("40%", "50%") }, "≤50%"),
                            JSX.createElement("td", { title: titleColumn("50%", "60%") }, "≤60%"),
                            JSX.createElement("td", { title: titleColumn("60%", "70%") }, "≤70%"),
                            JSX.createElement("td", { title: titleColumn("70%", "80%") }, "≤80%"),
                            JSX.createElement("td", { title: titleColumn("80%", "90%") }, "≤90%"),
                            JSX.createElement("td", { title: "Population dont la complétude de la note est strictement supérieure à 90%, mais pas totale." }, "<100%"),
                            JSX.createElement("td", { title: "Population dont la complétude de la note est totale, égale à 100%." }, "100%")),
                        JSX.createElement("tr", null,
                            JSX.createElement("th", null, "Population"),
                            this.cplUserCells)));
                    const onchange = (ev) => {
                        const input = ev.target;
                        this[input.id] = parseInt(input.value) / 100;
                        this.refreshScoreStats();
                    };
                    sr.append(JSX.createElement("div", { id: "cplBounds" },
                        "Statistiques pour une complétude comprise entre",
                        JSX.createElement("input", { id: "cplMin", type: "number", value: "0", min: "0", max: "99", step: "1", onchange: onchange }),
                        "% (exclus) et",
                        JSX.createElement("input", { id: "cplMax", type: "number", value: "100", min: "1", max: "100", step: "1", onchange: onchange }),
                        "% (inclus)"));
                }
                this.infoStats = JSX.createElement("div", { id: "infoStats" },
                    JSX.createElement("div", { id: "scoreBounds", class: "stat" },
                        JSX.createElement("span", { class: "label" }, "Bornes de cette note"),
                        JSX.createElement("span", { id: "minMax" },
                            "[",
                            this.def.min,
                            "-",
                            this.def.max,
                            "]")),
                    JSX.createElement("div", { id: "usersStat", class: "stat" },
                        JSX.createElement("span", { class: "label" }, "Echantillon"),
                        JSX.createElement("span", { id: "users" },
                            this.countUsers,
                            " / ",
                            this.totalUsers)));
                this.scoreStats = JSX.createElement("div", { id: "scoresStats" },
                    JSX.createElement("div", { id: "minStat", class: "stat" },
                        JSX.createElement("span", { class: "label" }, "Note la plus basse"),
                        this.scoreMin),
                    JSX.createElement("div", { id: "maxStat", class: "stat" },
                        JSX.createElement("span", { class: "label" }, "Note la plus élevée"),
                        this.scoreMax),
                    JSX.createElement("div", { id: "medStat", class: "stat" },
                        JSX.createElement("span", { class: "label" }, "Note médiane"),
                        this.scoreMed),
                    JSX.createElement("div", { id: "avrgStat", class: "stat" },
                        JSX.createElement("span", { class: "label" }, "Note moyenne"),
                        this.scoreAvrg));
                this.scoreRowTbl = JSX.createElement("tr", null,
                    JSX.createElement("th", null, "Note"));
                this.scoreUsersRowTbl = JSX.createElement("tr", null,
                    JSX.createElement("th", null, "Population"));
                this.scoreTbl = JSX.createElement("table", { id: "scoresTbl" },
                    this.scoreRowTbl,
                    this.scoreUsersRowTbl);
                sr.append(this.infoStats, this.scoreStats, this.scoreTbl);
            }
            this.refreshStats();
        }
    }
    async refreshStats() {
        await backend.scores.fetchScoresFor([this.def.id], this.usersScore);
        this.refreshCplStats();
        this.refreshScoreStats();
    }
    refreshCplStats() {
        if (!this.cplUserCells)
            return;
        const scoreId = this.def.id;
        const datas = [];
        for (let i = 0; i < 12; i++)
            datas[i] = [];
        for (const us of this.usersScore) {
            const score = us.scores.get(scoreId);
            if (score && isScoreDataComputable(score) && typeof score.c === "number") {
                if (score.c === 1) {
                    datas[11].push(us.user.pseudo);
                }
                else {
                    datas[Math.ceil(score.c * 10)].push(`${us.user.pseudo} : ${numToStr(score.c)}%`);
                }
            }
            else {
                datas[0].push(us.user.pseudo);
            }
        }
        for (let i = 0; i < 12; i++) {
            const td = this.cplUserCells[i];
            const d = datas[i];
            td.textContent = d.length > 0 ? d.length.toString() : null;
            td.title = d.join("\n");
        }
    }
    refreshScoreStats() {
        const scoreId = this.def.id;
        const users = [];
        const scores = [];
        for (const us of this.usersScore) {
            const score = us.scores.get(scoreId);
            if (score && isScoreDataComputable(score)) {
                if (typeof score.c === "number" && (score.c <= this.cplMin || score.c > this.cplMax))
                    continue;
                users.push(us.user.pseudo);
                scores.push(score.s);
            }
        }
        this.totalUsers.textContent = this.usersScore.length.toString();
        this.countUsers.textContent = users.length.toString();
        if (users.length === 0) {
            this.scoreStats.hidden = true;
            this.scoreTbl.hidden = true;
        }
        else {
            this.scoreStats.hidden = false;
            this.scoreTbl.hidden = false;
            let min = Number.MAX_SAFE_INTEGER;
            let max = Number.MIN_SAFE_INTEGER;
            let sum = 0;
            let discret = (this.def.max - this.def.min) < 20;
            for (const s of scores) {
                if (s < min)
                    min = s;
                if (s > max)
                    max = s;
                sum += s;
                if (discret && Math.round(s) !== s)
                    discret = false;
            }
            this.scoreMin.textContent = numToStr(min);
            this.scoreMax.textContent = numToStr(max);
            this.scoreAvrg.textContent = numToStr(sum / users.length);
            const med = scores.concat().sort();
            if (med.length % 2 === 0) {
                const i = med.length / 2;
                this.scoreMed.textContent = numToStr((med[i - 1] + med[i]) / 2);
            }
            else {
                this.scoreMed.textContent = numToStr(med[Math.floor(med.length / 2)]);
            }
            clearTds(this.scoreRowTbl);
            clearTds(this.scoreUsersRowTbl);
            const datas = [];
            if (discret) {
                for (let i = this.def.min; i <= this.def.max; i++) {
                    this.scoreRowTbl.appendChild(JSX.createElement("td", null, i));
                }
                for (let i = 0; i <= this.def.max - this.def.min; i++)
                    datas[i] = [];
                for (let i = 0; i < scores.length; i++) {
                    datas[scores[i] - this.def.min].push(users[i]);
                }
            }
            else {
                const delta = this.def.max - this.def.min;
                let step;
                if (delta <= 1) {
                    step = .1;
                }
                else if (delta <= 5) {
                    step = .5;
                }
                else if (delta <= 20) {
                    step = 1;
                }
                else {
                    let count = delta;
                    step = 1;
                    while (count > 20) {
                        if (isInt(delta / (step * 2)))
                            step *= 2;
                        else if (isInt(delta / (step * 5)))
                            step *= 5;
                        else if (isInt(delta / (step * 3)))
                            step *= 3;
                        else
                            step = count / 20;
                        count = Math.round(delta / step);
                    }
                }
                const count = Math.round(delta / step);
                let prevScore = null;
                for (let i = 1; i < count; i++) {
                    const curScore = numToStr(this.def.min + (step * i));
                    this.scoreRowTbl.appendChild(JSX.createElement("td", { title: this.titleColumn(prevScore, curScore) },
                        "≤",
                        curScore));
                    prevScore = curScore;
                }
                const curScore = numToStr(this.def.max);
                this.scoreRowTbl.appendChild(JSX.createElement("td", { title: this.titleColumn(prevScore, curScore) },
                    "≤",
                    curScore));
                for (let i = 0; i < Math.round((this.def.max - this.def.min) / step); i++)
                    datas[i] = [];
                for (let i = 0; i < scores.length; i++) {
                    datas[Math.floor((scores[i] - this.def.min) / step)].push(`${users[i]} : ${numToStr(scores[i])}`);
                }
            }
            for (const d of datas) {
                this.scoreUsersRowTbl.appendChild(JSX.createElement("td", { title: d.join("\n") }, d.length));
            }
        }
    }
    titleColumn(prevScroe, curScore) {
        if (prevScroe)
            return `Population avec une note comprise entre ${prevScroe} (exclus) et ${curScore} (inclus).`;
        return `Population avec une note inférieure ou égale à ${curScore}.`;
    }
}
function numToStr(n) {
    return (Math.round(n * 100) / 100).toString();
}
function clearTds(tr) {
    let ch = tr.firstElementChild;
    while (ch) {
        const next = ch.nextElementSibling;
        if (ch.localName === "td")
            ch.remove();
        ch = next;
    }
}
function isInt(n) { return Math.round(n) === n; }
customElements.define('p-scoresstats', ScoreStats);
