import { JSX, ShadowJsx } from "./jsx.js";
import { backend, currentUserMode } from "./webapp.js";
import { DIALOG } from "./dialog.js";
import { styleLinkTag } from "./commons.js";
export function isScoreDefValuable(def) { return typeof def.min === "number"; }
export function isScoreDefValueEditable(def) { return def.editMode && isScoreDefValuable(def); }
export function isScoreDefCompletion(def) { return isScoreDefValuable(def) && def.type === "Comput"; }
export function isScoreDefAnnotable(def) { return (def === null || def === void 0 ? void 0 : def.annotMaxLen) > 0; }
function isScoreEditable(def) {
    if (isScoreDefValueEditable(def) || isScoreDefAnnotable(def)) {
        switch (def.type) {
            case "Instr":
                return backend.scores.scoreStoreMode === "server" ? currentUserMode.asCorrector : true;
            case "Self":
                return backend.scores.scoreStoreMode === "server" ? backend.user && !currentUserMode.asCorrector : true;
            case "Scorm":
                return true;
        }
    }
    return false;
}
export class ScoreConfig extends Array {
    constructor(normDisp) {
        super();
        this.normDisp = normDisp;
    }
    addDisplay(disp) {
        this.push(disp);
        return this;
    }
    buildSticker(parent, def, score, label) {
        const caller = parent.host;
        const isEditable = isScoreEditable(def);
        let needPopup = isEditable;
        if (parent.redrawRoot)
            parent.redrawRoot.textContent = null;
        else {
            caller.classList.add(def.type);
            if (label)
                parent.appendChild(JSX.createElement("span", { class: "label" }, label));
            parent.redrawRoot = parent.appendChild(JSX.createElement("span", { class: "sticker" }));
        }
        parent.redrawRoot.classList.toggle("editable", isEditable);
        if (currentUserMode.asCorrector && backend.scores.isScoresInSynthAvailable() && !caller.userInSynth) {
            needPopup = true;
            caller.setAttribute("state", isEditable ? "correctorEditable" : "corrector");
            parent.redrawRoot.appendChild(JSX.createElement("span", { class: "toCorrect" }, isEditable ? "Corriger..." : "Consulter..."));
        }
        else {
            const normedS = this.normScore(score, def);
            for (const disp of this) {
                if (!disp.matcher || (typeof (normedS === null || normedS === void 0 ? void 0 : normedS.s) === "number" && disp.matcher(normedS === null || normedS === void 0 ? void 0 : normedS.s))) {
                    if (disp.buildSticker(parent, def, normedS, isEditable, label) === 'shortened')
                        needPopup = true;
                    break;
                }
            }
        }
        if (needPopup) {
            if (!caller.classList.contains("popupable")) {
                caller.classList.add("popupable");
                caller.onpointerdown = () => {
                    this.buildPopup(caller, def, label);
                };
            }
        }
        else if (caller.classList.contains("popupable")) {
            caller.classList.remove("popupable");
            caller.onpointerdown = null;
        }
    }
    async buildPopup(caller, def, label) {
        var _a, _b;
        if ((_a = caller.getAttribute("state")) === null || _a === void 0 ? void 0 : _a.startsWith("corrector")) {
            await import("./scoreSynth.js");
            DIALOG.showModal(JSX.createElement("p-scoressynth", { "score-id": def.id }), caller);
            return;
        }
        await backend.scores.waitScoresAvailable();
        const isEdition = isScoreEditable(def);
        const rootPopup = JSX.createElement("div", { class: `scorePopup ${def.type}` },
            JSX.createElement(ShadowJsx, null, await styleLinkTag(isEdition ? "scoreEdit.css" : "score.css")));
        const root = rootPopup.shadowRoot;
        if (isEdition) {
            const score = caller.scoreInSynth || backend.scores.fetchedScores.get(def.id);
            const form = root.appendChild(JSX.createElement("form", null));
            if (isScoreDefValueEditable(def)) {
                switch (def.editMode) {
                    case "button": {
                        const radios = form.appendChild(JSX.createElement("fieldset", { class: "scoreButtons" }));
                        const curr = score === null || score === void 0 ? void 0 : score.s;
                        if (def.pairs) {
                            for (const p of def.pairs) {
                                radios.appendChild(JSX.createElement("label", null,
                                    JSX.createElement("input", { name: "v", type: "radio", value: p.value, checked: p.value === curr ? "" : undefined }),
                                    p.label || p.value));
                            }
                        }
                        else {
                            for (let i = def.min; i <= def.max; i++) {
                                radios.appendChild(JSX.createElement("label", null,
                                    JSX.createElement("input", { name: "v", type: "radio", value: i, checked: i === curr ? "" : undefined }),
                                    i.toString()));
                            }
                        }
                        radios.appendChild(JSX.createElement("button", { onclick: (ev) => {
                                form.elements.namedItem("v").forEach((input) => input.checked = false);
                                ev.preventDefault();
                            } }, "Effacer"));
                        break;
                    }
                    case "enum": {
                        const select = form.appendChild(JSX.createElement("select", { name: "v", class: "scoreEnum" }));
                        const curr = score === null || score === void 0 ? void 0 : score.s;
                        if (def.pairs) {
                            for (const p of def.pairs) {
                                select.appendChild(JSX.createElement("option", { value: p.value, selected: p.value === curr ? "" : undefined }, p.label || p.value));
                            }
                        }
                        else {
                            for (let i = def.min; i <= def.max; i++) {
                                select.appendChild(JSX.createElement("option", { value: i, selected: i === curr ? "" : undefined }, i.toString()));
                            }
                        }
                        select.append(JSX.createElement("br", null), JSX.createElement("option", { value: "", selected: curr == null ? "" : undefined }, "Non renseignée"));
                        break;
                    }
                    default: {
                        form.appendChild(JSX.createElement("div", { class: "scoreInput" },
                            JSX.createElement("input", { name: "v", type: "number", value: (_b = score === null || score === void 0 ? void 0 : score.s) !== null && _b !== void 0 ? _b : undefined, min: def.min, max: def.max, step: def.editMode === '2' ? ".01" : def.editMode === 'half' ? ".5" : def.editMode === 'quarter' ? ".25" : def.editMode === '1' ? ".1" : "1", onblur: function () {
                                    const v = this.value;
                                    if (v === "")
                                        return;
                                    const num = parseFloat(v);
                                    if (num < def.min)
                                        this.value = def.min.toString();
                                    if (num > def.max)
                                        this.value = def.max.toString();
                                } }),
                            JSX.createElement("span", { class: "scoreBounds" }, def.min != 0 ? `[${def.min} > def.max]` : `/${def.max}`)));
                    }
                }
            }
            if (isScoreDefAnnotable(def)) {
                form.append(JSX.createElement("label", { class: "annotField" },
                    JSX.createElement("span", null, def.annotLabel),
                    JSX.createElement("textarea", { name: "a", maxlength: def.annotMaxLen }, (score === null || score === void 0 ? void 0 : score.a) || "")));
            }
            root.appendChild(JSX.createElement("footer", null,
                JSX.createElement("button", { id: "cancel", onclick: function () { var _a; (_a = DIALOG.findDialog(this)) === null || _a === void 0 ? void 0 : _a.close(); } }, "Annuler"),
                JSX.createElement("button", { id: "save", onclick: () => { this.doSave(def, score, form, caller.userInSynth); } }, "Enregister")));
            DIALOG.showModal(rootPopup, caller);
            setTimeout(() => { var _a; (_a = form.querySelector("input,select,textarea")) === null || _a === void 0 ? void 0 : _a.focus(); }, 100);
        }
        else {
            const score = this.normScore(caller.scoreInSynth || backend.scores.fetchedScores.get(def.id), def);
            for (const disp of this) {
                if (!disp.matcher || (typeof (score === null || score === void 0 ? void 0 : score.s) === "number" && disp.matcher(score === null || score === void 0 ? void 0 : score.s))) {
                    disp.buildPopup(root, def, score, label);
                    break;
                }
            }
            DIALOG.showPopup(rootPopup, caller);
        }
    }
    normScore(s, def) {
        const score = s ? newScoreFromStore(s, def.min, def.max) : newScore(def.min, undefined, def.max);
        if (this.normDisp)
            this.normDisp(score);
        return score;
    }
    async doSave(def, score, form, forUser) {
        var _a, _b, _c;
        const v = (_a = form.elements.namedItem("v")) === null || _a === void 0 ? void 0 : _a.value;
        const a = ((_b = form.elements.namedItem("a")) === null || _b === void 0 ? void 0 : _b.value) || undefined;
        const s = ((v === null || v === void 0 ? void 0 : v.length) > 0) ? parseFloat(v) : undefined;
        await backend.scores.setScore(def.id, { s, a, c: score === null || score === void 0 ? void 0 : score.c }, forUser);
        (_c = DIALOG.findDialog(form)) === null || _c === void 0 ? void 0 : _c.close();
    }
}
class ScoreDisplay {
    constructor(matcher) {
        this.matcher = matcher;
    }
    setIcon(path) {
        this.iconPath = path;
        return this;
    }
    setStep(r) {
        this.step = Math.max(0, r);
        return this;
    }
    str(n) {
        if (isNaN(n) || n == null)
            return "";
        if (!this.step)
            return Math.floor(n).toString();
        return (Math.floor(n / this.step) * this.step).toString();
    }
    buildSticker(parent, def, score, isEditable, label) {
        const root = parent.redrawRoot;
        if (score === undefined) {
            parent.host.setAttribute("state", "loading");
        }
        else if (score === null || (score.s == null && score.a == null)) {
            parent.host.setAttribute("state", isEditable ? "missingEditable" : "missing");
            root.appendChild(JSX.createElement("span", { class: "missing" }, isEditable ? isScoreDefValuable(def) ? "Noter..." : "Commenter..." : " "));
        }
        else {
            parent.host.setAttribute("state", isEditable ? "filledEditable" : "filled");
            if (this.iconPath)
                root.appendChild(JSX.createElement("img", { src: this.iconPath }));
            this.buildQuantifiedScore(root, score);
            if (typeof score.c === "number")
                this.appendCompletion(root, def, score);
            return this.appendShortAnnot(root, def, score);
        }
    }
    buildPopup(parent, def, score, label) {
        const root = parent.appendChild(JSX.createElement("div", { class: "scoreFull" }));
        if (score == null || (score.s == null && score.a == null)) {
            root.appendChild(JSX.createElement("span", { class: "missing" }));
        }
        else {
            if (this.iconPath)
                root.appendChild(JSX.createElement("img", { src: this.iconPath }));
            this.buildQuantifiedScore(root, score);
            if (typeof score.c === "number")
                this.appendCompletion(root, def, score);
            this.appendFullAnnot(root, def, score);
        }
    }
    appendShortAnnot(root, def, score) {
        if (isScoreDefAnnotable(def) && score.a) {
            let shortened;
            let annot = score.a;
            if (annot.length > 12) {
                annot = annot.substring(0, 9) + "...";
                shortened = 'shortened';
            }
            root.appendChild(JSX.createElement("span", { class: "annot" }, annot));
            return shortened;
        }
    }
    appendCompletion(root, def, score) {
        const percent = Math.round(score.c * 100);
        root.append(JSX.createElement("span", { class: "completion", title: `Complétude de la note : ${percent}%` },
            JSX.createElement("span", { class: "cplDone", style: `flex:${score.c} ${score.c} auto` }),
            JSX.createElement("span", { class: "cplMissing", style: `flex:${1 - score.c} ${1 - score.c} auto` })));
    }
    appendFullAnnot(root, def, score) {
        if (score.a && isScoreDefAnnotable(def)) {
            root.append(JSX.createElement("div", { class: "annotLabel" }, def.annotLabel), JSX.createElement("div", { class: "annot" }, score.a));
        }
    }
}
export class NumScoreDisplay extends ScoreDisplay {
    setForm(f) {
        this.form = f;
        return this;
    }
    buildQuantifiedScore(root, score) {
        if (score.s == null)
            return;
        switch (this.form) {
            case "score":
                root.appendChild(JSX.createElement("span", { class: "score Alone" },
                    JSX.createElement("span", { class: "value" }, this.str(score.s))));
                break;
            case "scoreMinMax":
                root.appendChild(JSX.createElement("span", { class: "score MinMax" },
                    JSX.createElement("span", { class: "value" }, this.str(score.s)),
                    JSX.createElement("span", { class: "scoreStartRg" }, " ["),
                    JSX.createElement("span", { class: "scoreMin" }, this.str(score.m)),
                    JSX.createElement("span", { class: "scoreInRg" }, " > "),
                    JSX.createElement("span", { class: "scoreMax" }, this.str(score.M)),
                    JSX.createElement("span", { class: "scoreEndRg" }, "]")));
                break;
            default:
                root.appendChild(JSX.createElement("span", { class: "score Max" },
                    JSX.createElement("span", { class: "value" }, this.str(score.s)),
                    JSX.createElement("span", { class: "scoreSp" }, " / "),
                    JSX.createElement("span", { class: "scoreMax" }, this.str(score.M))));
                break;
        }
    }
}
export class FreeScoreDisplay extends ScoreDisplay {
    setText(t) {
        this.text = t;
        return this;
    }
    buildQuantifiedScore(root, score) {
        if (score.s == null)
            return;
        if (this.text)
            root.appendChild(JSX.createElement("span", { class: "score Free" }, this.text(this.str(score.s), this.str(score.m), this.str(score.M))));
    }
}
export function isScoreComputable(s) { return s && typeof s.s === "number" && !isNaN(s.s); }
export const isScoreDataComputable = isScoreComputable;
export function newScoreFromStore(data, min, max) {
    return { m: min, s: data.s, M: max, c: data.c, a: data.a };
}
export function newScore(min, s, max, c = undefined, a = undefined) {
    return { m: min, s: s, M: max, c: c, a: a };
}
export function cloneScore(s) {
    return { m: s.m, s: s.s, M: s.M, c: s.c, a: s.a };
}
export function fixupScore(s) {
    if (typeof s.s !== "number" || isNaN(s.s) || typeof s.m !== "number" || isNaN(s.m)) {
        s.s = undefined;
    }
    else if (s.s < s.m) {
        s.s = s.m;
    }
    else if (s.s > s.M) {
        s.s = s.M;
    }
    if (typeof s.c !== undefined) {
        if (typeof s.c !== "number" || isNaN(s.c))
            s.c = undefined;
        else if (s.c > 1)
            s.c = 1;
        else if (s.c < 0)
            s.c = 0;
    }
}
export function normScale(s, min, max, aroundZero = false) {
    if (isScoreComputable(s)) {
        if (aroundZero && min <= 0 && max >= 0) {
            if (s.s > 0) {
                s.s = s.s / s.M * max;
            }
            else if (s.s < 0) {
                s.s = s.s / s.m * min;
            }
        }
        else {
            s.s = (s.s - s.m) / (s.M - s.m) * (max - min) + min;
        }
    }
    s.m = min;
    s.M = max;
}
export function coef(s, coef) {
    s.m *= coef;
    s.M *= coef;
    if (isScoreComputable(s))
        s.s *= coef;
}
export function truncateMin(s, min) {
    s.m = min;
    if (isScoreComputable(s) && s.s < min)
        s.s = min;
}
export function truncateMax(s, max) {
    s.M = max;
    if (isScoreComputable(s) && s.s > max)
        s.s = max;
}
export function roundStep(s, step) {
    if (isScoreComputable(s))
        s.s = Math.round(s.s / step) * step;
}
export function computeOnChange(defsModule, def, score, oldScores) {
    const result = new Map();
    result.set(def, score);
    function getScore(scoreId) {
        const def = defsModule.getScoreDef(scoreId);
        const data = result.get(def) || oldScores.get(scoreId);
        const s = data ? newScoreFromStore(data, def.min, def.max) : newScore(def.min, undefined, def.max);
        if (typeof def.coef === "number")
            coef(s, def.coef);
        fixupScore(s);
        return s;
    }
    function computeCallers(def) {
        if (def.usedBy)
            for (const id of def.usedBy) {
                const caller = defsModule.getScoreDef(id);
                const newScore = caller.compute(getScore);
                const old = oldScores.get(caller.id);
                if (!isScoreValuesEquals(newScore, old)) {
                    newScore.a = old === null || old === void 0 ? void 0 : old.a;
                    result.set(caller, newScore);
                }
                computeCallers(caller);
            }
    }
    computeCallers(def);
    return result;
}
function isScoreValuesEquals(s1, s2) {
    if (s1 == null)
        return s2 == null;
    if (s2 == null)
        return false;
    return s1.s === s2.s && s1.c === s2.c;
}
export function makeScoresSet(useIds, getScore) {
    var _a;
    const result = [];
    for (const subSet of useIds) {
        for (const id of subSet.ids) {
            const s = getScore(id);
            (_a = subSet.normEach) === null || _a === void 0 ? void 0 : _a.call(subSet, s);
            result.push(s);
        }
    }
    return result;
}
export function sumScores(set) {
    if (set == null || set.length === 0)
        return null;
    let sum = 0;
    let min = 0;
    let max = 0;
    let cpl = 0;
    for (const s of set) {
        if (isScoreComputable(s)) {
            if (typeof s.c === "number") {
                cpl += (s.M - s.m) * s.c;
                sum += s.s * s.c;
            }
            else {
                sum += s.s;
                cpl += (s.M - s.m);
            }
        }
        min += s.m;
        max += s.M;
    }
    if (cpl === 0)
        return null;
    const c = cpl / (max - min);
    return newScore(min, sum / c, max, c);
}
export function setCompletion(fulMin, fullMax, to) {
    to.c = (to.M - to.m) / (fullMax - fulMin);
}
