import { ScormViewer } from "./scorm-viewer.js";
import { backend, currentUserMode } from "./webapp.js";
import { JSX } from "./jsx.js";
export class ScormPlayer extends ScormViewer {
    async viewShown(resView) {
        var _a;
        if (currentUserMode.asCorrector && backend.scorm.isScormSynthAvailable && !((_a = this.scoOptions) === null || _a === void 0 ? void 0 : _a.forLearner)) {
            return this.buildSynth(resView);
        }
        else {
            return super.viewShown(resView);
        }
    }
    async showNode(node, iframe) {
        var _a, _b;
        const scoId = this.getAttribute("res-id");
        if (scoId) {
            if (this.scoOpening === scoId)
                return;
            this.scoOpening = scoId;
            backend.onScoChanged = (scoKey, event) => {
                if (scoKey === scoId)
                    this.showNode(node, iframe);
            };
            try {
                const opened = await backend.scorm.open(scoId, this.scoOptions);
                if (this.userMsg) {
                    this.userMsg.parentNode.removeChild(this.userMsg);
                    this.userMsg = null;
                }
                if (!opened) {
                    this.userMsg = backend.scorm.buildMsgNoStorageAvailable(JSX.createElement("span", { class: "msg noStore" }));
                    this.shadowRoot.getElementById("body").insertAdjacentElement("beforebegin", this.userMsg);
                }
                else {
                    const msg = (_b = (_a = (window.API_1484_11 || window.API)) === null || _a === void 0 ? void 0 : _a.getRestrictionUserMsg) === null || _b === void 0 ? void 0 : _b.call(window.API_1484_11 || window.API);
                    if (msg) {
                        this.userMsg = JSX.createElement("span", { class: "msg noStore" }, msg);
                        this.shadowRoot.getElementById("body").insertAdjacentElement("beforebegin", this.userMsg);
                    }
                }
            }
            finally {
                this.scoOpening = null;
            }
        }
        return super.showNode(node, iframe);
    }
    disconnectedCallback() {
        var _a, _b;
        backend.onScoChanged = null;
        (_a = window.API_1484_11) === null || _a === void 0 ? void 0 : _a.Terminate("");
        (_b = window.API) === null || _b === void 0 ? void 0 : _b.LMSFinish("");
    }
    async buildSynth(resView) {
        await this._init();
        await import("./scormSynth.js");
        this.view = this.shadowRoot.appendChild(JSX.createElement("scorm-synth", { href: resView.getAttribute("href"), "res-id": this.getAttribute("res-id"), "res-title": this.getAttribute("res-title") || "", "sc-manifest": this.hasAttribute("sc-manifest") ? "" : undefined }));
    }
}
customElements.define('scorm-player', ScormPlayer);
