import { styleLinkTag } from "./commons.js";
import { JSX } from "./jsx.js";
import { getDeepUrl, listenChangeUrl } from "./views.js";
export class ScormViewer extends HTMLElement {
    async viewShown(resView) {
        const resPath = resView.getAttribute("href");
        if (resPath)
            await this.start(resPath);
    }
    async _init() {
        var _a;
        if (!this.shadowRoot) {
            this.attachShadow({ mode: "open" }).append(await styleLinkTag("scormViewer.css"));
        }
        else {
            (_a = this.view) === null || _a === void 0 ? void 0 : _a.remove();
        }
    }
    async start(resPath) {
        await this._init();
        const sr = this.shadowRoot;
        const loading = sr.appendChild(JSX.createElement("div", { class: "msg loading" }, "Chargement..."));
        if (!resPath.endsWith('/'))
            resPath += "/";
        this.rootScormUrl = new URL(resPath, window.location.href);
        try {
            if (this.hasAttribute("sc-manifest")) {
                const resp = await fetch(new URL("imsScormCam.xml", this.rootScormUrl));
                if (resp.ok)
                    this.buildMapScManifest(new DOMParser().parseFromString(await resp.text(), "application/xml"));
            }
            else {
                const resp = await fetch(new URL("imsmanifest.xml", this.rootScormUrl));
                if (resp.ok)
                    this.buildMap(new DOMParser().parseFromString(await resp.text(), "application/xml"));
            }
        }
        catch (e) {
            console.log(e);
        }
        loading.remove();
        if (this.mapNodes) {
            const iframe = sr.appendChild(JSX.createElement("iframe", { id: "body" }));
            this.view = iframe;
            if (this.mapNodes.length > 1 || this.mapNodes[0].subNodes) {
                console.log(":::TODO scorm MAP");
            }
            else {
                await this.showNode(this.mapNodes[0], iframe);
                listenChangeUrl(iframe);
            }
        }
        else {
            sr.appendChild(JSX.createElement("div", { class: "msg error" }, "Aucun contenu trouvé dans cette ressource SCORM"));
        }
    }
    showNode(node, iframe) {
        iframe.src = getDeepUrl(new URL(node.href, this.rootScormUrl).href, iframe, this.rootScormUrl.href);
    }
    buildMapScManifest(manifest) {
        this.mapNodes = [];
        for (const r of manifest.documentElement.querySelectorAll("item")) {
            this.mapNodes.push({ title: r.getAttribute("title"), href: r.getAttribute("href") });
        }
    }
    buildMap(manifest) {
        const scoMap = new Map();
        for (const r of manifest.querySelectorAll("manifest > resources > resource[href]")) {
            scoMap.set(r.getAttribute("identifier"), r.getAttribute("href"));
        }
        const orgs = manifest.querySelector("manifest > organizations");
        if (orgs) {
            const orgId = orgs.getAttribute("default");
            let org = null;
            for (let ch = orgs.firstElementChild; ch; ch = ch.nextElementSibling) {
                if (!orgId || ch.getAttribute("identifier") === orgId) {
                    org = ch;
                    break;
                }
            }
            if (org) {
                this.mapNodes = [];
                for (let ch = org.firstElementChild; ch; ch = ch.nextElementSibling) {
                    if (ch.localName === "item")
                        this.mapNodes.push(this.buildNode(ch, scoMap));
                }
                if (this.mapNodes.length === 0)
                    this.mapNodes = null;
            }
        }
    }
    buildNode(item, scoMap) {
        const res = { title: this.extractTitle(item) };
        res.href = scoMap.get(item.getAttribute("identifierref"));
        for (let ch = item.firstElementChild; ch; ch = ch.nextElementSibling) {
            if (ch.localName === "item")
                (res.subNodes || (res.subNodes = [])).push(this.buildNode(ch, scoMap));
        }
        return res;
    }
    extractTitle(item) {
        for (let ch = item.firstElementChild; ch; ch = this.nextElementSibling)
            if (ch.localName === "title")
                return ch.textContent;
        return "";
    }
}
customElements.define('scorm-viewer', ScormViewer);
