export class ScormAPIBase {
    constructor() {
        this.lastError = "0";
        this.credit = 'credit';
        this.mode = 'normal';
        this.noStore = false;
    }
    initScorm(options) {
        if (options) {
            if (options.mode)
                this.mode = options.mode;
            if (options.credit)
                this.credit = options.credit;
            if ('noStore' in options)
                this.noStore = options.noStore;
        }
    }
    Initialize(p) {
        this.api = '2k4';
        return "true";
    }
    LMSInitialize(p) {
        this.api = '12';
        return "true";
    }
    Terminate(p) {
        if (this.api === '2k4')
            return this.doTerminate();
        return 'false';
    }
    LMSFinish(p) {
        if (this.api === '12')
            return this.doTerminate();
        return 'false';
    }
    GetValue(key) {
        this.lastError = "0";
        const m = _getters2k4[key];
        if (m) {
            const r = this[m]();
            return r == null ? "" : r.toString();
        }
        const fields = key.split(".");
        const len = fields.length;
        switch (fields[len - 1]) {
            case "_version":
                return "1.0";
            case "_children": {
                const result = _children2k4[fields[len - 2]];
                if (!result) {
                    this.lastError = "301";
                    return "";
                }
                return result;
            }
            case "_count":
                return this.doCountFields(fields).toString();
            default:
                const v = this.getOther(key, fields);
                if (v == null) {
                    return "";
                }
                return v;
        }
    }
    LMSGetValue(key) {
        this.lastError = "0";
        const m = _getters12[key];
        if (m) {
            const r = this[m]();
            return r == null ? "" : r.toString();
        }
        const fields = key.split(".");
        const len = fields.length;
        switch (fields[len - 1]) {
            case "_version":
                return "1.0";
            case "_children": {
                const result = _children2k4[fields[len - 2]];
                if (!result) {
                    this.lastError = "301";
                    return "";
                }
                return result;
            }
            case "_count":
                return this.doCountFields(fields).toString();
            default:
                const v = this.getOther(key, fields);
                if (v == null) {
                    this.lastError = "403";
                    return "";
                }
                return v;
        }
    }
    SetValue(key, value) {
        this.lastError = "0";
        const m = _setters2k4[key];
        if (m)
            return this[m](value);
        return this.setOther(key, value) ? "true" : "false";
    }
    LMSSetValue(key, value) {
        const m = _setters12[key];
        if (m)
            return this[m](value);
        return this.setOther(key, value) ? "true" : "false";
    }
    Commit(p) {
        this.lastError = "0";
        if (!this.noStore)
            this.doCommit();
        return "true";
    }
    LMSCommit(p) { return this.Commit(p); }
    GetLastError() { return this.lastError; }
    LMSGetLastError() { return this.lastError; }
    GetErrorString(keyError) {
        if (keyError === "0")
            return "";
        return keyError || "";
    }
    LMSGetErrorString(keyError) { return this.GetErrorString(keyError); }
    GetDiagnostic() { return this.GetErrorString(this.lastError); }
    LMSGetDiagnostic() { return this.GetDiagnostic(); }
    getCredit() { return this.credit; }
    getMode() { return this.mode; }
    setSessionTime(v) {
        this.lastSessionTime = v;
    }
    doTerminate() {
        if (this.lastSessionTime)
            this.setSessionTimeOnTerminate(this.lastSessionTime);
        const r = this.Commit("");
        this.api = null;
        return r;
    }
    serializeMsDuration(timeInMs) {
        if (timeInMs == null)
            timeInMs = 0;
        if (this.api === "2k4") {
            const h = timeInMs > 3600000 ? Math.floor(timeInMs / 3600000) : 0;
            const m = Math.floor(timeInMs / 60000) % 60;
            const s = Math.floor(timeInMs / 1000) % 60;
            const cs = Math.round(timeInMs / 10) % 100;
            const sStr = cs != 0 ? `${s}.${cs}` : s.toString();
            let time = "PT";
            if (h > 0)
                time = time + h + "H";
            if (m > 0)
                time = time + m + "M";
            if (sStr !== "0" || time == "PT")
                time = time + sStr + "S";
            return time;
        }
        else {
            const h = timeInMs > 3600000 ? Math.floor(timeInMs / 3600000) : 0;
            const m = Math.floor(timeInMs / 60000) % 60;
            const s = Math.floor(timeInMs / 1000) % 60;
            const cs = Math.round(Math.floor(timeInMs % 1000) / 10);
            return (h < 10 ? "0" + h : h) + ":" + (m < 10 ? "0" + m : m) + ":" + (s < 10 ? "0" + s : s) + "." + (cs < 10 ? "0" + cs : cs);
        }
    }
    parseDurationInMs(time) {
        if (!time)
            return 0;
        if (this.api === "2k4") {
            const parts = /P(?:(\d+)Y)?(?:(\d+)M)?(?:(\d+)D)?(?:T(?:(\d+)H)?(?:(\d+)M)?(?:([\d.]+)S)?)?/.exec(time);
            const y = parseInt(parts[1], 10) || 0;
            const M = parseInt(parts[2], 10) || 0;
            const d = parseInt(parts[3], 10) || 0;
            const h = parseInt(parts[4], 10) || 0;
            const m = parseInt(parts[5], 10) || 0;
            const s = parseFloat(parts[6]) || 0;
            return y * 31536000000 + M * 2628000000 + d * 86400000 + h * 3600000 + m * 3600 + Math.round(s * 100) * 10;
        }
        else {
            const parts = time.split(":");
            return parseInt(parts[0], 10) * 3600000 + parseInt(parts[1], 10) + 60000 + Math.round(parseFloat(parts[2]) * 1000);
        }
    }
}
const _children2k4 = {
    "comments_from_learner": "comment,location,timestamp",
    "comments_from_lms": "comment,location,timestamp",
    "interactions": "id,type,objectives,timestamp,correct_responses,weighting,learner_response,result,latency,description",
    "learner_preference": "audio_level,language,delivery_speed,audio_captioning",
    "objectives": "id,score,success_status,completion_status,description",
    "score": "scaled,raw,min,max"
};
const _children12 = Object.create(_children2k4);
_children12["core"] = "student_id,student_name,lesson_location,credit,lesson_status,entry,score,total_time,exit,session_time";
_children12["score"] = "raw,min,max";
const _setters12 = {
    "cmi.core.lesson_location": "setLocation",
    "cmi.core.lesson_status": "setStatus",
    "cmi.core.session_time": "setSessionTime",
    "cmi.core.exit": "setExit",
    "cmi.core.score.raw": "setScoreRaw",
    "cmi.core.score.min": "setScoreMin",
    "cmi.core.score.max": "setScoreMax"
};
const _setters2k4 = {
    "cmi.location": "setLocation",
    "cmi.completion_status": "setStatus",
    "cmi.session_time": "setSessionTime",
    "cmi.exit": "setExit",
    "cmi.score.raw": "setScoreRaw",
    "cmi.score.min": "setScoreMin",
    "cmi.score.max": "setScoreMax",
    "cmi.score.scaled": "setScoreScaled"
};
const _getters12 = {
    "cmi.core.lesson_location": "getLocation",
    "cmi.core.lesson_status": "getStatus",
    "cmi.core.total_time": "getTotalTime",
    "cmi.core.exit": "getExit",
    "cmi.core.lesson_mode": "getMode",
    "cmi.core.credit": "getCredit",
    "cmi.core.score.raw": "getScoreRaw",
    "cmi.core.score.min": "getScoreMin",
    "cmi.core.score.max": "getScoreMax"
};
const _getters2k4 = {
    "cmi.location": "getLocation",
    "cmi.completion_status": "getStatus",
    "cmi.total_time": "getTotalTime",
    "cmi.exit": "getExit",
    "cmi.mode": "getMode",
    "cmi.credit": "getCredit",
    "cmi.score.raw": "getScoreRaw",
    "cmi.score.min": "getScoreMin",
    "cmi.score.max": "getScoreMax",
    "cmi.score.scaled": "getScoreScaled"
};
