import { backend, shadowRoots } from "./webapp.js";
import { makeButton, styleLinkTag } from "./commons.js";
import { JSX, ShadowJsx } from "./jsx.js";
import { TabButton } from "./tab-button.js";
import { DIALOG } from "./dialog.js";
import { ScormPlayer } from "./scorm-player.js";
import { escapeCsv } from "./appCommons.js";
export class ScormSynth extends HTMLElement {
    get resUrl() { return this.getAttribute("href"); }
    get resId() { return this.getAttribute("res-id"); }
    get resTitle() { return this.getAttribute("res-title") || ""; }
    async connectedCallback() {
        const sr = this.attachShadow({ mode: "open" });
        shadowRoots.add(sr);
        sr.append(await styleLinkTag("scormSynth.css"));
        const msg = sr.appendChild(JSX.createElement("div", { class: "msg" }, "Chargement des données..."));
        const cohort = await backend.fetchCohort();
        const usersScorm = cohort.users.map((u) => new SynthUserScormInst(u)).sort((u1, u2) => u1.user.pseudo.localeCompare(u2.user.pseudo));
        const resId = this.resId;
        await backend.scorm.fetchScormSynthFor([resId], usersScorm);
        msg.remove();
        const table = JSX.createElement("scorm-synth-table", { id: "details" });
        table.synthUserScorm = usersScorm;
        table.resId = this.resId;
        table.resUrl = this.resUrl;
        table.resTitle = this.resTitle;
        table.scManifest = this.hasAttribute("sc-manifest");
        const synthExport = JSX.createElement("scorm-synth-export", { id: "exportBtn" }, "Exporter en CSV");
        synthExport.synthUserScorm = usersScorm;
        synthExport.resId = this.resId;
        sr.append(JSX.createElement("nav", { role: "tablist" },
            JSX.createElement(TabButton, { id: "detailsBtn", role: "tab", "aria-controls": "details", "aria-selected": "true" }, "Détails"),
            JSX.createElement("div", { class: "showScormCtn" },
                JSX.createElement("a", { id: "showScorm", onclick: () => { table.onShowScormFor(); } }, "Visualiser"))), table, JSX.createElement("div", { class: "footer" }, synthExport));
    }
    disconnectedCallback() {
        shadowRoots.delete(this.shadowRoot);
    }
}
customElements.define('scorm-synth', ScormSynth);
export class SynthUserScormInst {
    constructor(user) {
        this.user = user;
    }
}
export class ScormSynthTable extends HTMLElement {
    connectedCallback() {
        var _a, _b;
        let scoreMin = undefined;
        let scoreMax = undefined;
        let scoreScaled = false;
        let progress = false;
        for (const u of this.synthUserScorm) {
            const d = (_a = u.scormStats) === null || _a === void 0 ? void 0 : _a.get(this.resId);
            if (d) {
                if (typeof d.scoreMin === 'number') {
                    if (scoreMin === undefined)
                        scoreMin = d.scoreMin;
                    else if (scoreMin !== d.scoreMin)
                        scoreMin = null;
                    if (typeof d.scoreMax === 'number') {
                        if (scoreMax === undefined)
                            scoreMax = d.scoreMax;
                        else if (scoreMax !== d.scoreMax)
                            scoreMax = null;
                    }
                }
                if (!scoreScaled && typeof d.scoreScaled === 'number')
                    scoreScaled = true;
                if (!progress && typeof d.progress === 'number')
                    progress = true;
            }
        }
        const header = this.appendChild(JSX.createElement("tr", null,
            JSX.createElement("th", null,
                JSX.createElement("div", null, backend.isSendMsgAvailable() ? JSX.createElement("button", { class: "sendMsg", id: "sendMsgToCohort", onclick: () => { this.onSenMsgToCohort(); }, title: "Envoyer un message à tous" }) : undefined)),
            JSX.createElement("th", null,
                JSX.createElement("div", null, "Statut"))));
        if (scoreMin !== undefined)
            header.appendChild(JSX.createElement("th", null,
                JSX.createElement("div", null, `Note [${scoreMin === null ? "min" : scoreMin}, ${scoreMin == null ? "max" : scoreMax}]`)));
        if (scoreScaled)
            header.appendChild(JSX.createElement("th", null,
                JSX.createElement("div", null, "Note (%)")));
        if (progress)
            header.appendChild(JSX.createElement("th", null,
                JSX.createElement("div", null, "Progression (%)")));
        const step = scoreMin != null ? computeStep(scoreMin, scoreMax) : undefined;
        for (const u of this.synthUserScorm) {
            const d = (_b = u.scormStats) === null || _b === void 0 ? void 0 : _b.get(this.resId);
            const tr = this.appendChild(JSX.createElement("tr", null,
                JSX.createElement("th", null,
                    JSX.createElement("span", null,
                        JSX.createElement("button", { class: "user", onclick: () => { this.onShowScormFor(u); } }, u.user.pseudo),
                        backend.isSendMsgAvailable() ? JSX.createElement("button", { class: "sendMsg", onclick: () => { this.onSenMsg(u); }, title: "Envoyer un message" }) : undefined)),
                JSX.createElement("td", null,
                    JSX.createElement("span", null, statusLabel.get((d === null || d === void 0 ? void 0 : d.status) || "")))));
            if (scoreMin !== undefined) {
                const cell = tr.appendChild(JSX.createElement("td", null));
                if (typeof (d === null || d === void 0 ? void 0 : d.scoreRaw) === "number") {
                    if (scoreMin == null)
                        cell.appendChild(JSX.createElement("span", null,
                            JSX.createElement("span", { class: "scoreRaw" }, scoreRawToStr(d.scoreRaw, computeStep(d.scoreMin, d.scoreMax))),
                            JSX.createElement("span", { class: "scoreBound" },
                                "[",
                                d.scoreMin,
                                ", ",
                                d.scoreMax,
                                "]")));
                    else
                        cell.appendChild(JSX.createElement("span", { class: "scoreRaw" }, scoreRawToStr(d.scoreRaw, step)));
                }
            }
            if (scoreScaled) {
                const cell = tr.appendChild(JSX.createElement("td", null));
                if (typeof (d === null || d === void 0 ? void 0 : d.scoreScaled) === "number")
                    cell.appendChild(JSX.createElement("span", { class: "scoreScaled" },
                        Math.round(d.scoreScaled * 100),
                        " %"));
            }
            if (progress) {
                const cell = tr.appendChild(JSX.createElement("td", null));
                if (typeof (d === null || d === void 0 ? void 0 : d.progress) === "number")
                    cell.appendChild(JSX.createElement("span", { class: "progress" },
                        Math.round(d.progress * 100),
                        " %"));
            }
        }
    }
    async onShowScormFor(u) {
        const viewer = new ScormPlayer();
        if (this.scManifest)
            viewer.setAttribute("sc-manifest", "");
        viewer.setAttribute("res-id", this.resId);
        viewer.scoOptions = u ? { noStore: true, credit: "no-credit", mode: "review", forLearner: u.user } : {};
        DIALOG.showModal(JSX.createElement("div", { style: "width:90vw; height:90vh; display:flex; flex-direction:column; padding:0;" },
            JSX.createElement(ShadowJsx, null,
                await styleLinkTag("dialog.css"),
                JSX.createElement("header", { style: "display:flex; align-items:center;" },
                    JSX.createElement("h3", { style: "flex:1; margin:.5rem;" }, this.resTitle),
                    JSX.createElement("button", { class: "close", title: "Fermer", onclick: function (ev) {
                            DIALOG.findDialog(this).close();
                        } })),
                JSX.createElement("main", { style: "display:flex;flex-direction:column; flex:1" }, viewer))));
        viewer.start(this.resUrl);
    }
    onSenMsg(u) {
        backend.sendMsgTo({ users: [u.user] }, this.resTitle);
    }
    onSenMsgToCohort() {
        backend.sendMsgTo({ users: this.synthUserScorm.map(userSynth => userSynth.user) }, this.resTitle);
    }
}
customElements.define('scorm-synth-table', ScormSynthTable);
function computeStep(min, max) {
    const d = (max - min) / 100;
    if (d < .001)
        return .0001;
    else if (d < .01)
        return .001;
    else if (d < .1)
        return .01;
    else if (d < 1)
        return .1;
    return 1;
}
function scoreRawToStr(raw, step) {
    return (Math.round(raw / step) * step).toString();
}
const statusLabel = new Map();
statusLabel.set("", "");
statusLabel.set("unknown", "Inconnu");
statusLabel.set("incomplete", "En cours");
statusLabel.set("completed", "Achevé");
statusLabel.set("not attempted", "Non commencé");
statusLabel.set("browsed", "Consulté");
statusLabel.set("passed", "Réussi");
statusLabel.set("failed", "Achevé en échec");
export class ScormSynthExport extends HTMLElement {
    constructor() {
        super();
        makeButton(this, "Exporter les données Scorm");
    }
    onClick(ev) {
        var _a;
        const link = document.createElement('a');
        const data = ['"Utilisateur";"Statut";"Note min";"Note";"Note max";"Note [0,1]";"Progression [0,1]"\r\n'];
        for (const us of this.synthUserScorm) {
            data.push(escapeCsv(us.user.pseudo));
            const d = (_a = us.scormStats) === null || _a === void 0 ? void 0 : _a.get(this.resId);
            if (d) {
                data.push(";");
                if (d.status)
                    data.push(d.status);
                data.push(";");
                if (typeof d.scoreMin == "number")
                    data.push(d.scoreMin.toString());
                data.push(";");
                if (typeof d.scoreRaw == "number")
                    data.push(d.scoreRaw.toString());
                data.push(";");
                if (typeof d.scoreMax == "number")
                    data.push(d.scoreMax.toString());
                data.push(";");
                if (typeof d.scoreScaled == "number")
                    data.push(d.scoreScaled.toString());
                data.push(";");
                if (typeof d.progress == "number")
                    data.push(d.progress.toString());
                data.push("\r\n");
            }
            else {
                data.push(";;;;;;\r\n");
            }
        }
        const url = URL.createObjectURL(new Blob(data));
        link.href = url;
        link.download = "scorm.csv";
        link.click();
        setTimeout(function () { URL.revokeObjectURL(url); }, 0);
    }
}
customElements.define('scorm-synth-export', ScormSynthExport);
