import { authenticating, currentUserMode, injectDynStates, isTimeReached, openPage } from "./webapp.js";
import { makeButton } from "./commons.js";
import { JSX } from "./jsx.js";
import { showNotNowDialog } from "./widgets.js";
import { QS_deepUrl, QS_keyView, ResView } from "./views.js";
export const tabs = {};
const qs = new URLSearchParams(location.search);
export function switchTab(tab, hash, initing) {
    currentTab = tab;
    for (let key in tabs) {
        const t = tabs[key];
        if (t === tab) {
            qs.set("t", t.codeTab);
            if (t.pg)
                qs.set("pg", t.pg);
            else
                qs.delete("pg");
            if (!initing) {
                qs.delete(QS_keyView);
                qs.delete(QS_deepUrl);
            }
            try {
                window.popstateFromUs = true;
                location.hash = hash || "";
                t.activate(hash);
                if (initing)
                    window.history.replaceState(null, "", hash ? `?${qs}#${hash}` : `?${qs}`);
                else
                    window.history.pushState(null, "", hash ? `?${qs}#${hash}` : `?${qs}`);
            }
            finally {
                window.popstateFromUs = false;
            }
        }
        else
            t === null || t === void 0 ? void 0 : t.deactivate();
    }
}
export let currentTab;
export async function initFromLocation() {
    const hash = location.hash;
    const qs = new URLSearchParams(location.search);
    const tabCd = qs.get("t");
    if (!tabCd)
        return false;
    const tab = tabs[tabCd];
    if (!tab)
        return false;
    const pg = qs.get("pg") || "";
    if (tab === tabs.sub) {
        const ok = await openPage(pg, false) === void 0;
        if (ok)
            switchTab(tab, hash === null || hash === void 0 ? void 0 : hash.substring(1), true);
        return ok;
    }
    tab.openAtInit();
    return true;
}
window.addEventListener("popstate", (ev) => {
    if (!window.popstateFromUs)
        initFromLocation();
});
export class Tab extends HTMLElement {
    constructor() {
        super();
        this.pg = null;
        makeButton(this);
    }
    activate(hash) {
        this.setAttribute("active", "");
        if (this.bodyTab)
            this.bodyTab.hidden = false;
    }
    deactivate() {
        this.removeAttribute("active");
        if (this.bodyTab)
            this.bodyTab.hidden = true;
    }
    matchHref(href) {
        return this.pg == href.split(/\?/)[0];
    }
    getBodyTab() {
        if (!this.bodyTab)
            this.bodyTab = document.getElementById("bodies").appendChild(JSX.createElement("div", null));
        return this.bodyTab;
    }
}
export class HomeTab extends Tab {
    get codeTab() { return "home"; }
    constructor() {
        super();
        tabs.home = this;
    }
    initHome() {
        this.bodyTab = document.getElementById("home");
    }
    async update(timedRef, logicRef) {
        if (this.pg === "")
            return;
        this.pg = "";
        const { Desk } = await import("./desk.js");
        this.desk = this.getBodyTab().appendChild(new Desk());
    }
    onClick(ev) {
        openPage(this.codeTab);
    }
    openAtInit() {
        openPage(this.codeTab, true, true);
    }
}
customElements.define('tab-home', HomeTab);
export class ProgTab extends Tab {
    get codeTab() { return "prog"; }
    constructor() {
        super();
        tabs.prog = this;
    }
    async update(timedRef, logicRef) {
        if (this.pg === timedRef)
            return;
        this.pg = timedRef;
        this.getBodyTab().innerHTML = await (await fetch(new URL(timedRef, document.baseURI))).text();
        injectDynStates(this.getBodyTab());
    }
    onClick(ev) {
        openPage(this.getAttribute("href"));
    }
    openAtInit() {
        openPage(this.getAttribute("href"), true, true);
    }
}
customElements.define('tab-prog', ProgTab);
export class SubTab extends Tab {
    get codeTab() { return "sub"; }
    constructor() {
        super();
        tabs.sub = this;
        this.hidden = true;
    }
    activate(hash) {
        this.hidden = false;
        super.activate(hash);
    }
    async update(timedRef, logicRef) {
        if (authenticating)
            return;
        if (this.timedRef === timedRef) {
            this.redraw();
            return;
        }
        this.timedRef = timedRef;
        this.pg = logicRef;
        if (!this.shadowRoot) {
            this.attachShadow({ mode: 'open' }).append(JSX.createElement("button", { part: "close-btn", onclick: this.onClose, title: "Fermer" }), JSX.createElement("slot", null));
        }
        this.textContent = null;
        this.getBodyTab().innerHTML = await (await fetch(new URL(timedRef, document.baseURI))).text();
        injectDynStates(this.getBodyTab());
        const h1 = this.getBodyTab().getElementsByTagName("h1").item(0);
        if (h1)
            for (let ch = h1.firstChild; ch; ch = ch.nextSibling) {
                if (ch instanceof HTMLElement && (ch.classList.contains("icon") || ch.localName.indexOf('-') > 0))
                    continue;
                this.appendChild(this.deepClone(ch));
            }
        if (!this.hasChildNodes())
            this.textContent = "Ressource";
        this.resViews = this.getBodyTab().getElementsByTagName("res-views").item(0);
        this.redraw();
        this.hidden = false;
    }
    deepClone(n) {
        const clone = n.cloneNode(false);
        if (clone instanceof HTMLElement)
            clone.style.color = "unset";
        for (let ch = n.firstChild; ch; ch = ch.nextSibling) {
            if (ch.localName === "p-info")
                continue;
            clone.appendChild(this.deepClone(ch));
        }
        return clone;
    }
    async refresh() {
        if (this.pg)
            await openPage(this.pg, false);
    }
    redraw() {
        if (!this.resViews)
            return;
        const showSolDt = this.resViews.getAttribute("show-sol");
        if (showSolDt) {
            const showSol = currentUserMode.solVisible || isTimeReached(showSolDt);
            for (let view = this.resViews.firstElementChild; view; view = view.nextElementSibling) {
                if (view instanceof ResView) {
                    const solPol = view.getAttribute("sol-pol");
                    let unavailable;
                    if (view.classList.contains("sol")) {
                        if (showSol) {
                            unavailable = false;
                        }
                        else {
                            unavailable = solPol !== "always";
                        }
                    }
                    else {
                        unavailable = solPol === "replaceAfter" ? showSol : false;
                    }
                    view.classList.toggle("unavailable", unavailable);
                    if (unavailable) {
                        view.onclick = () => {
                            showNotNowDialog({ ti: view.getAttribute("title"), vis: showSolDt });
                        };
                    }
                    else {
                        view.onclick = view.onClick;
                    }
                }
            }
        }
        this.resViews.selectViewAtInit();
    }
    onClose(ev) {
        ev.stopImmediatePropagation();
        const me = this.getRootNode().host;
        me.hidden = true;
        me.timedRef = null;
        me.pg = null;
        me.textContent = null;
        me.getBodyTab().textContent = null;
        tabs.prog.click();
    }
    onClick(ev) {
        openPage(this.pg);
    }
    openAtInit() {
        openPage(this.pg, true, true);
    }
}
customElements.define('tab-sub', SubTab);
export class CalTab extends Tab {
    get codeTab() { return "cal"; }
    constructor() {
        super();
        tabs.cal = this;
    }
    async update(timedRef, logicRef) {
        if (this.pg === "")
            return;
        this.pg = "";
        const { Calendar } = await import("./calendar.js");
        this.getBodyTab().append(new Calendar());
    }
    onClick(ev) {
        openPage(this.codeTab);
    }
    openAtInit() {
        openPage(this.codeTab, true, true);
    }
}
customElements.define('tab-cal', CalTab);
export class MoreTab extends Tab {
    get codeTab() { return this.getAttribute("code-tab"); }
    constructor() {
        super();
        tabs[this.codeTab] = this;
    }
    async update() {
        if (this.pg === "")
            return;
        this.pg = "";
        this.getBodyTab().innerHTML = await (await fetch(new URL(this.getAttribute("href"), document.baseURI))).text();
    }
    onClick(ev) {
        openPage(this.codeTab);
    }
    openAtInit() {
        openPage(this.codeTab, true, true);
    }
}
customElements.define('tab-more', MoreTab);
