import { makeButton } from "./commons.js";
let initing = false;
export class ResViews extends HTMLElement {
    connectedCallback() {
        if (!("timedDatas" in window)) {
            Promise.resolve().then(() => {
                this.selectViewAtInit();
            });
        }
    }
    selectViewAtInit() {
        let count = 0;
        let view = null;
        const keyView = new URLSearchParams(location.search).get(QS_keyView);
        for (let v = this.firstElementChild; v; v = v.nextElementSibling) {
            if (!v.classList.contains("unavailable") && v instanceof ResView) {
                count++;
                if (!view)
                    view = v;
                if (keyView && v.getAttribute("v-key") === keyView)
                    view = v;
            }
        }
        this.hidden = count <= 1;
        view === null || view === void 0 ? void 0 : view.showAtInit();
    }
}
customElements.define('res-views', ResViews);
export class ResView extends HTMLElement {
    get selected() { return this.hasAttribute("selected"); }
    constructor() {
        super();
        makeButton(this);
    }
    async setSelected(on) {
        const target = this.getAttribute("target");
        if (on) {
            this.setAttribute("selected", "");
            for (let o = this.previousElementSibling; o; o = o.previousElementSibling)
                if (o instanceof ResView)
                    o.setSelected(false);
            for (let o = this.nextElementSibling; o; o = o.nextElementSibling)
                if (o instanceof ResView)
                    o.setSelected(false);
            if (target) {
                const frm = document.getElementById(target);
                if (frm) {
                    frm.hidden = false;
                    listenChangeUrl(frm);
                    window.open(getDeepUrl(this.getAttribute("href"), frm), target);
                }
            }
            else {
                const jsLib = this.getAttribute("js-lib");
                if (jsLib)
                    await import(new URL(jsLib, import.meta.url).href);
                const body = this.getBodyView();
                if (body) {
                    if (body.viewShown)
                        await body.viewShown(this);
                    else if (body instanceof HTMLIFrameElement) {
                        listenChangeUrl(body);
                        body.contentWindow.location.href = getDeepUrl(body.src, body);
                    }
                    body.hidden = false;
                }
            }
            updateWindowQS(QS_keyView, this.getAttribute("v-key"));
        }
        else {
            this.removeAttribute("selected");
            if (!target) {
                const body = this.getBodyView();
                if (body) {
                    body.hidden = true;
                    if (body.viewHidden)
                        body.viewHidden(this);
                }
            }
            else {
                const frm = document.getElementById(target);
                if (frm)
                    frm.hidden = true;
            }
        }
    }
    getBodyView() { return document.getElementById(this.getAttribute("view-id")); }
    onClick(ev) {
        if (!this.selected)
            this.setSelected(true);
    }
    async showAtInit() {
        try {
            initing = true;
            await this.setSelected(true);
        }
        finally {
            initing = false;
        }
    }
}
customElements.define('res-view', ResView);
export function listenChangeUrl(frm) {
    if (!frm.listenUrl) {
        frm.listenUrl = true;
        const lstn = (ev) => {
            let subWin;
            if (ev.type === "hashchange") {
                subWin = ev.target;
            }
            else if (ev.type === "load") {
                subWin = ev.target.contentWindow;
                subWin.addEventListener("hashchange", lstn);
            }
            if (subWin.frameElement.clientWidth > 0) {
                const base = frm.getAttribute("base-url");
                let newUrl = subWin.location.href;
                if (base && newUrl.startsWith(base))
                    newUrl = newUrl.substring(base.length);
                updateWindowQS(QS_deepUrl, newUrl);
            }
        };
        frm.addEventListener("load", lstn);
    }
}
function updateWindowQS(param, value) {
    try {
        window.popstateFromUs = true;
        const qs = new URLSearchParams(location.search);
        qs.set(param, value);
        window.history.replaceState(null, "", `?${qs}`);
    }
    finally {
        window.popstateFromUs = false;
    }
}
export function getDeepUrl(primaryUrl, frm, rootUrl) {
    let baseUrl = rootUrl || primaryUrl;
    if (baseUrl) {
        const url = new URL(baseUrl);
        url.search = "";
        url.hash = "";
        const lastSlash = url.pathname.lastIndexOf('/');
        if (lastSlash > 0)
            url.pathname = url.pathname.substring(0, lastSlash + 1);
        else
            url.pathname += '/';
        baseUrl = url.href;
    }
    frm.setAttribute("base-url", baseUrl);
    if (initing) {
        const u = new URLSearchParams(location.search).get(QS_deepUrl);
        if (u)
            try {
                return new URL(u, baseUrl).href;
            }
            catch (e) { }
    }
    return primaryUrl;
}
export const QS_keyView = "v";
export const QS_deepUrl = "u";
