import { initFromLocation, switchTab, tabs } from "./tabs.js";
import "./widgets.js";
import { JSX } from "./jsx.js";
import { BackendFacade } from "./backendFacade.js";
import { DIALOG } from "./dialog.js";
import { onHideBars } from "./commons.js";
export async function openPage(href, focus = true, initing = false) {
    let logicRef = href;
    let i = href.indexOf("?");
    if (i > 0)
        logicRef = href.substring(0, i);
    let hash = null;
    i = href.indexOf("#");
    if (i > 0)
        hash = href.substring(i + 1);
    let tab = tabFromFromLogicRef(logicRef);
    if (!tab)
        return null;
    let timedRef = logicRef;
    const visObj = visByLogicUrl.get(logicRef);
    if (visObj && !currentUserMode.contentVisible && !isTimeReached(visObj.vis))
        return visObj;
    if (tab === tabs.sub) {
        const d = resByLogicUrl.get(logicRef);
        if (d && (currentUserMode.solVisible || isTimeReached(d.sol)))
            timedRef = d.solUrl;
    }
    await tab.update(timedRef, logicRef);
    if (focus)
        switchTab(tab, hash, initing);
}
let resolveDataAvailable;
export const datasAvailable = new Promise((resolve) => { resolveDataAvailable = resolve; });
export function getBlocSol(solId) {
    const s = byId.get(solId);
    return (s === null || s === void 0 ? void 0 : s.T) === "blocSol" ? s : null;
}
export function getBlocSolUrl(s) {
    return (s === null || s === void 0 ? void 0 : s.T) === "blocSol" && (currentUserMode.solVisible || isTimeReached(s.sol)) ? s.url : null;
}
export function getInProgUrl(objId) {
    const s = byId.get(objId);
    if (!(s === null || s === void 0 ? void 0 : s.vis))
        return null;
    const t = s === null || s === void 0 ? void 0 : s.T;
    if (t !== "sea" && t !== "act")
        return null;
    return s && (currentUserMode.contentVisible || isTimeReached(s.vis)) ? s.inProgUrl : null;
}
export function getInProgDt(objId) {
    var _a;
    return (_a = byId.get(objId)) === null || _a === void 0 ? void 0 : _a.vis;
}
export function refresh() {
    refreshDynStates('p-sol,p-seainprog,tab-sub,p-desk,user-mode,p-cal,p-a,p-injectinprog', (tag) => { tag.refresh(); });
    refreshDynStates("[p-rendid]", (tag) => {
        const rendId = tag.getAttribute("p-rendid");
        setRendState(tag, backend.rend.isValidated(rendId), isRendLimitReached(rendId));
    });
}
export function evtDt(d) {
    return d.vis || d.sol || d.limit;
}
export function parentOf(d) {
    return byId.get(d.p);
}
export function timedData(id) { return byId.get(id); }
export function isDataAct(d) { return d.T === "act"; }
export function isDataSea(d) { return (d === null || d === void 0 ? void 0 : d.T) === "sea"; }
export function isDataTimedPeriod(d) { return d.T === "seq" || d.T === "prog"; }
export function isTimeReached(time) {
    return tsToLocalDt(now()) >= time;
}
export function isRendLimitReached(rendId) {
    var _a, _b;
    const limit = (_b = (_a = timedData(rendId)) === null || _a === void 0 ? void 0 : _a.rend) === null || _b === void 0 ? void 0 : _b.limit;
    return limit ? isTimeReached(limit) : false;
}
export function bgColor(d) {
    var _a;
    return window.timedDatas.colors[(_a = (d || d).sc) !== null && _a !== void 0 ? _a : d.c];
}
export function borderColor(d) {
    var _a;
    return window.timedDatas.colors[(_a = (d || d).tc) !== null && _a !== void 0 ? _a : d.c];
}
export function icon(d) {
    return window.timedDatas.icons[d.ico];
}
export function injectDynStates(root) {
    for (const tag of root.querySelectorAll("[p-rendid]")) {
        const rendId = tag.getAttribute("p-rendid");
        setRendState(tag, backend.rend.isValidated(rendId), isRendLimitReached(rendId));
    }
    for (let s of root.getElementsByTagName("p-score")) {
        s.refreshScore(backend.scores.fetchedScores);
    }
}
export const shadowRoots = new Set();
export function refreshDynStates(query, cb, inModals = false) {
    for (const tag of document.body.querySelectorAll(query))
        cb(tag);
    for (const sr of shadowRoots) {
        for (const tag of sr.querySelectorAll(query))
            cb(tag);
    }
}
export function refreshScores(scores) {
    refreshDynStates("p-score", (s) => {
        s.refreshScore(scores);
    });
}
export function refreshScoresFor(user, scores) {
    refreshDynStates("p-score", (s) => {
        s.refreshScoreFor(user, scores);
    }, true);
}
export function setRendState(tag, rendValid, limitReached) {
    const isValidated = rendValid != null;
    tag.classList.toggle("validated", isValidated);
    tag.classList.toggle("locked", limitReached);
    const ts = rendValid ? rendValid.clientTs || rendValid.ts : undefined;
    if (ts) {
        tag.title = `Rendu validé le ${new Date(ts).toLocaleString()}`;
    }
    else if (rendValid) {
        tag.title = `Rendu validé`;
    }
    else {
        tag.title = limitReached ? `Rendu clos non validé` : `Rendu à valider`;
    }
    const date = tag.getAttribute("date");
    if (date) {
        tag.textContent = `${limitReached ? "clos" : date} `;
        tag.appendChild(JSX.createElement("span", { class: "rendSt" }, isValidated ? "validé" : "non validé"));
    }
}
export function buildSeance(d, start, end, fmt) {
    const sea = JSX.createElement("div", { class: "evt sea", style: `border-color:${borderColor(d)}` },
        JSX.createElement("span", { class: "ico", style: `background-color:${bgColor(d)}; background-image:url(${icon(d)})` }),
        buildTi(d),
        JSX.createElement("span", { class: "dt" }, fmt.formatRange(start, end)));
    sea.addEventListener("click", () => { openPage(d.url); });
    return sea;
}
export function buildRend(d, limit, fmt) {
    const isLimitReached = limit ? limit.getTime() < now() : false;
    const isValidated = backend.rend.isValidated(d.id);
    const dt = JSX.createElement("span", { class: "dt limit", "p-rendid": d.id, style: `border-color:${bgColor(d.act)}`, date: fmt.format(limit) });
    setRendState(dt, isValidated, isLimitReached);
    return JSX.createElement("p-rendbutton", { class: "evt rend", style: `border-color:${borderColor(d.act)}`, href: d.url, css: `${window.skinRoot}rend.css` },
        JSX.createElement("span", { class: "ico", style: `background-color:${bgColor(d.act)}; background-image:url(${icon(d.act)})` }),
        buildTi(d.act),
        dt);
}
export function buildAct(d) {
    const act = JSX.createElement("div", { class: "evt sea", style: `border-color:${borderColor(d)}` },
        JSX.createElement("span", { class: "ico", style: `background-color:${bgColor(d)}; background-image:url(${icon(d)})` }),
        buildTi(d));
    act.addEventListener("click", () => { openPage(d.url); });
    return act;
}
export function buildTi(d) {
    const ti = JSX.createElement("span", { class: "ti" });
    ti.innerHTML = d.ti;
    return ti;
}
export function formatDt(isoDt, fmt) {
    try {
        return fmt.format(new Date(isoDt));
    }
    catch (e) { }
    return isoDt;
}
export const timeFmt = Intl.DateTimeFormat(undefined, { hour: "numeric", minute: "numeric" });
export const dateTimeFmt = Intl.DateTimeFormat(undefined, { weekday: 'short', month: 'short', day: 'numeric', hour: 'numeric', minute: 'numeric' });
export const dateFmt = Intl.DateTimeFormat(undefined, { weekday: 'short', month: 'short', day: 'numeric' });
export const dateShortTimeFmt = Intl.DateTimeFormat(undefined, { month: 'short', day: 'numeric', hour: 'numeric', minute: 'numeric' });
export const dateShortFmt = Intl.DateTimeFormat(undefined, { month: 'short', day: 'numeric' });
export const dateYearFmt = Intl.DateTimeFormat(undefined, { year: "numeric", month: 'short', day: 'numeric' });
export function tsToLocalDt(dt) {
    const t = new Date(dt);
    t.setMinutes(t.getMinutes() - t.getTimezoneOffset());
    return t.toISOString().substring(0, 16);
}
export function now() { return nowTs !== null && nowTs !== void 0 ? nowTs : Date.now(); }
export function setNowDate(date) {
    nowTs = (date === null || date === void 0 ? void 0 : date.getTime()) || undefined;
}
export const backend = new BackendFacade();
class FrontendImpl {
    async onLogout() {
        if (!backend.rend.isSynchroAvailable()) {
            backend.resetUser();
            await backend.rend.clearLocalData();
            for (const d of currentRendDialog)
                await d.tryClose();
            DIALOG.closeAllModals();
            refresh();
            (tabs.home || tabs.prog).click();
        }
    }
    async onLogin() {
        return backend.onLoginChange();
    }
    get isUserActive() { return activeState.isUserActive; }
    get userActiveChangeListeners() { return activeState.changeListeners; }
    showDialog(dialog, modal) {
        document.body.appendChild(dialog);
        if (modal)
            dialog.showModal();
        else
            dialog.show();
    }
    listenOnHideBars(lstn) {
        onHideBars.add(lstn);
    }
    scoresUpdated(scores, user) {
        if (!user || user === backend.user)
            refreshScores(scores);
        else
            refreshScoresFor(user, scores);
    }
    scoChanged(scoKey, event) {
        var _a;
        (_a = backend.onScoChanged) === null || _a === void 0 ? void 0 : _a.call(null, scoKey, event);
    }
}
const frontend = new FrontendImpl();
let nowTs = undefined;
function tabFromFromLogicRef(ref) {
    for (const k in tabs)
        if (k == ref)
            return tabs[k];
    const start = ref.lastIndexOf('/');
    const end = ref.indexOf(".html", start);
    const pgKey = end > 0 ? ref.substring(start + 1, end) : "";
    if (pgKey === "program")
        return tabs.prog;
    return tabs.sub;
}
const byId = new Map();
const resByLogicUrl = new Map();
const visByLogicUrl = new Map();
window.addEventListener("load", async () => {
    var _a;
    (_a = tabs.home) === null || _a === void 0 ? void 0 : _a.initHome();
    await backend.initialize(frontend);
    function logicUrl(url) {
        const i = url.indexOf('?');
        return i > 0 ? url.substring(0, i) : url;
    }
    const timedDatas = window.timedDatas;
    for (const d of timedDatas.data) {
        if (d.start) {
            d.startOrig = d.start;
        }
        if (d.end) {
            d.endOrig = d.end;
        }
        if (d.id)
            byId.set(d.id, d);
        if (d.vis && d.url.indexOf('#') === -1) {
            const logicRef = logicUrl(d.url);
            if (tabFromFromLogicRef(logicRef) === tabs.sub)
                visByLogicUrl.set(logicRef, d);
        }
        if (d.rend) {
            const r = d.rend;
            r.act = d;
            r.id = d.id;
            if (r.act.vis)
                r.vis = r.act.vis;
        }
        if (d.res)
            for (const r of d.res) {
                byId.set(r.id, r);
                r.p = d.id;
                if (r.T === "res" && r.solUrl)
                    resByLogicUrl.set(logicUrl(r.url), r);
            }
    }
    resolveDataAvailable(timedDatas);
    authenticating = new Promise(async (resolver) => {
        try {
            if (!await initFromLocation())
                (tabs.home || tabs.prog).click();
            await backend.tryAuthenticate();
        }
        finally {
            authenticating = null;
            resolver();
        }
    });
});
export let authenticating = null;
export const currentUserMode = {
    contentVisible: false,
    solVisible: false,
    rendEditable: false,
    asCorrector: false
};
let currentRendDialog = [];
export function setCurrentRendDialog(area) {
    if (area)
        currentRendDialog.push(area);
    else
        currentRendDialog.length--;
}
export function scoreDefsModule() {
    if (!_scoresDefModule) {
        _scoresDefModule = import(new URL("scoresDef.js", window.location.href).href);
    }
    return _scoresDefModule;
}
let _scoresDefModule;
export const activeState = {
    isUserActive: true,
    changeListeners: new Set(),
    _mark: true,
    _lastMark: Date.now(),
    markAsUserActive() {
        if (!this._mark) {
            this._mark = true;
            this._lastMark = Date.now();
            if (!this.isUserActive) {
                this.isUserActive = true;
                this.changeListeners.forEach((cb) => cb(true));
            }
        }
    },
    _init() {
        const markActive = this.markAsUserActive.bind(this);
        window.addEventListener('mousemove', markActive, true);
        window.addEventListener('touchstart', markActive, true);
        window.addEventListener('keypdown', markActive, true);
        window.addEventListener('focus', markActive, true);
        document.addEventListener('visibilitychange', () => {
            if (document.visibilityState === 'visible')
                this.markAsUserActive();
        }, true);
        setInterval(() => {
            const now = Date.now();
            const delta = now - this._lastMark;
            if (this.isUserActive && delta > 180000) {
                this.isUserActive = false;
                this.changeListeners.forEach((cb) => cb(false));
            }
            this._mark = false;
        }, 10000);
    }
};
activeState._init();
