import { JSX, ShadowJsx } from "./jsx.js";
import { backend, currentUserMode, dateFmt, dateShortFmt, dateShortTimeFmt, dateTimeFmt, dateYearFmt, formatDt, getBlocSol, getBlocSolUrl, getInProgDt, getInProgUrl, injectDynStates, isTimeReached, openPage, refresh, scoreDefsModule, timeFmt } from "./webapp.js";
import { makeButton, styleLinkTag } from "./commons.js";
import { DIALOG } from "./dialog.js";
export class Link extends HTMLElement {
    constructor() {
        super();
        makeButton(this);
    }
    async onClick(ev) {
        const visObj = await openPage(this.getAttribute("href"));
        if (visObj)
            showNotNowDialog(visObj);
    }
}
customElements.define('p-link', Link);
let notNowDialog = null;
export function showNotNowDialog(visObj) {
    var _a, _b, _c;
    if (!notNowDialog) {
        notNowDialog = document.createElement("dialog");
        notNowDialog.id = "notNow";
        notNowDialog.addEventListener("click", function () { this.close(); });
        document.body.appendChild(notNowDialog);
    }
    const date = formatDt(visObj.vis, dateTimeFmt);
    const ti = JSX.createElement("h3", null);
    ti.innerHTML = (_c = (_a = visObj.ti) !== null && _a !== void 0 ? _a : (_b = visObj.act) === null || _b === void 0 ? void 0 : _b.ti) !== null && _c !== void 0 ? _c : "";
    notNowDialog.replaceChildren(ti, JSX.createElement("p", null, `Prochainement disponible : ${date}`));
    notNowDialog.showModal();
}
export class Sol extends HTMLElement {
    connectedCallback() {
        this.refresh();
    }
    async refresh() {
        const s = getBlocSol(this.getAttribute("id"));
        const url = getBlocSolUrl(s);
        if (url) {
            this.classList.toggle("unavailable", false);
            this.title = "";
            this.innerHTML = await (await fetch(url)).text();
            injectDynStates(this);
        }
        else if (s) {
            this.classList.toggle("unavailable", true);
            const date = formatDt(s.sol, dateTimeFmt);
            this.replaceChildren(JSX.createElement("span", { class: "unavailMsg" },
                "Les solutions seront visibles le ",
                date));
        }
    }
}
customElements.define('p-sol', Sol);
export class InjectInProg extends Sol {
    async refresh() {
        const id = this.getAttribute("id");
        const url = getInProgUrl(id);
        if (url) {
            this.classList.toggle("unavailable", false);
            this.innerHTML = await (await fetch(url)).text();
            injectDynStates(this);
        }
        else {
            this.classList.toggle("unavailable", true);
            const date = formatDt(getInProgDt(id), dateTimeFmt);
            this.replaceChildren(JSX.createElement("div", { class: "unavailMsg" },
                "Ce contenu sera visible le ",
                date));
        }
    }
}
customElements.define('p-injectinprog', InjectInProg);
export class DateTime extends HTMLElement {
    connectedCallback() {
        this.textContent = formatDt(this.getAttribute("date"), this.getFmt());
    }
    getFmt() {
        switch (this.getAttribute("format")) {
            case "time":
                return timeFmt;
            case "date":
                return dateFmt;
            case "dateTime":
                return dateTimeFmt;
            case "dateShort":
                return dateShortFmt;
            case "dateShortTime":
                return dateShortTimeFmt;
            case "dateYear":
                return dateYearFmt;
            default:
                return dateTimeFmt;
        }
    }
}
customElements.define('p-dt', DateTime);
export class DynATag extends HTMLElement {
    constructor() {
        super();
        makeButton(this);
    }
    connectedCallback() {
        this.refresh();
    }
    refresh() {
        const showSolDt = this.getAttribute("show-sol");
        const pol = this.getAttribute("sol-pol");
        const showSol = currentUserMode.solVisible || isTimeReached(showSolDt);
        if (this.classList.contains("sol")) {
            this.hidden = false;
            this.classList.toggle("unavailable", !showSol);
        }
        else {
            this.hidden = showSol && (pol === "replaceAfter" || pol === "after");
            this.classList.toggle("unavailable", false);
        }
    }
    onClick(ev) {
        this.refresh();
        if (this.hidden)
            return;
        if (this.classList.contains("unavailable")) {
            showNotNowDialog({ ti: this.getAttribute("title"), vis: this.getAttribute("show-sol") });
        }
        else {
            window.open(this.getAttribute("href"), this.getAttribute("target"));
        }
    }
}
customElements.define('p-a', DynATag);
export class UserMode extends HTMLElement {
    constructor() {
        super();
        makeButton(this);
    }
    refresh() {
        if (backend.isUserAdmin() || backend.isUserCorrector()) {
            this.hidden = false;
            if (currentUserMode.solVisible || currentUserMode.contentVisible || currentUserMode.rendEditable) {
                this.textContent = currentUserMode.asCorrector ? "Correction & supervision" : "Supervision";
            }
            else if (currentUserMode.asCorrector) {
                this.textContent = "Correction";
            }
            else {
                this.textContent = "Normal";
            }
        }
        else {
            this.hidden = true;
        }
    }
    async onClick(ev) {
        const onChange = (ev) => {
            const input = ev.target;
            currentUserMode[input.value] = input.checked;
            const store = Object.assign({ ts: Date.now() }, currentUserMode);
            localStorage === null || localStorage === void 0 ? void 0 : localStorage.setItem("par:currentUserMode", JSON.stringify(store));
            refresh();
        };
        const popup = DIALOG.showModal(JSX.createElement("div", null,
            JSX.createElement(ShadowJsx, null,
                await styleLinkTag("userMode.css"),
                JSX.createElement("button", { id: "close", title: "Fermer", onclick: () => popup.close() }),
                JSX.createElement("h3", null, "Modes d'affichages"),
                backend.isUserCorrector() ?
                    JSX.createElement("div", null,
                        JSX.createElement("h4", null, "Correction"),
                        JSX.createElement("label", null,
                            JSX.createElement("input", { type: "checkbox", value: "asCorrector", checked: currentUserMode.asCorrector || undefined, onchange: onChange }),
                            JSX.createElement("span", null, "Afficher les rendus et scores de tous")))
                    : undefined,
                backend.isUserAdmin() ?
                    JSX.createElement("div", null,
                        JSX.createElement("h4", null, "Supervision"),
                        JSX.createElement("label", null,
                            JSX.createElement("input", { type: "checkbox", value: "contentVisible", checked: currentUserMode.contentVisible || undefined, onchange: onChange }),
                            JSX.createElement("span", null, "Afficher tous les contenus")),
                        JSX.createElement("label", null,
                            JSX.createElement("input", { type: "checkbox", value: "solVisible", checked: currentUserMode.solVisible || undefined, onchange: onChange }),
                            JSX.createElement("span", null, "Afficher toutes les solutions")),
                        JSX.createElement("label", null,
                            JSX.createElement("input", { type: "checkbox", value: "rendEditable", checked: currentUserMode.rendEditable || undefined, onchange: onChange }),
                            JSX.createElement("span", null, "Préserver mes rendus éditables")))
                    : undefined)));
    }
}
customElements.define('user-mode', UserMode);
export class ScoreSticker extends HTMLElement {
    constructor() {
        super(...arguments);
        this.initing = true;
    }
    get scoreId() { return this.getAttribute("score-id"); }
    setUserScore(userScore) {
        var _a;
        this.scoreInSynth = ((_a = userScore.scores) === null || _a === void 0 ? void 0 : _a.get(this.scoreId)) || {};
        this.userInSynth = userScore.user;
        this.classList.add("inSynth");
        this.initing = false;
        this.redraw(this.scoreInSynth);
    }
    refreshScore(scores) {
        if (this.userInSynth)
            return;
        switch (backend.scores.scoreStoreMode) {
            case "initing":
            case "none":
                this.hidden = true;
                return;
        }
        if (this.getAttribute("vis-for") === "instr" && !currentUserMode.asCorrector) {
            this.hidden = true;
            return;
        }
        this.hidden = false;
        if (scores === null) {
            this.initing = true;
            this.redraw(undefined);
            return;
        }
        const score = scores === null || scores === void 0 ? void 0 : scores.get(this.scoreId);
        if (this.initing) {
            this.initing = false;
            this.redraw(score || null);
        }
        else if (score) {
            this.redraw(score);
        }
    }
    refreshScoreFor(user, scores) {
        if (this.userInSynth === user) {
            const newScore = scores.get(this.scoreId);
            if (newScore) {
                this.scoreInSynth = newScore;
                this.redraw(this.scoreInSynth);
            }
        }
    }
    async redraw(score) {
        if (!this.shadowRoot) {
            this.attachShadow({ mode: "open" });
            this.shadowRoot.appendChild(await styleLinkTag("score.css"));
        }
        const mod = await scoreDefsModule();
        await backend.scores.waitScoresAvailable();
        const def = mod.getScoreDef(this.scoreId);
        def.config.buildSticker(this.shadowRoot, def, score, this.getAttribute("label"));
    }
}
customElements.define('p-score', ScoreSticker);
